#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageaccount
version_added: "0.1.0"
short_description: Manage Azure storage accounts
description:
    - Create, update or delete a storage account.
options:
    auth_mode:
        description:
            - The mode in which to run the command. C(login) mode will directly use your login credentials for the authentication.
            - The legacy C(key) mode will attempt to query for an account key if no authentication parameters for the account are provided.
            - Can also be set via the environment variable C(AZURE_STORAGE_AUTH_MODE).
        default: key
        type: str
        choices:
            - key
            - login
        version_added: "3.10.0"
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
        aliases:
            - resource_group_name
    name:
        description:
            - Name of the storage account to update or create.
        type: str
        required: true
    state:
        description:
            - State of the storage account. Use C(present) to create or update a storage account and use C(absent) to delete an account.
            - C(failover) is used to failover the storage account to its secondary. This process can take up to a hour.
        default: present
        type: str
        choices:
            - absent
            - present
            - failover
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    account_type:
        description:
            - Type of storage account. Required when creating a storage account.
            - C(Standard_ZRS) and C(Premium_LRS) accounts cannot be changed to other account types.
            - Other account types cannot be changed to C(Standard_ZRS) or C(Premium_LRS).
        type: str
        choices:
            - Premium_LRS
            - Standard_GRS
            - Standard_LRS
            - Standard_RAGRS
            - Standard_ZRS
            - Premium_ZRS
            - Standard_RAGZRS
            - Standard_GZRS
        aliases:
            - type
    custom_domain:
        description:
            - User domain assigned to the storage account.
            - Must be a dictionary with I(name) and I(use_sub_domain) keys where I(name) is the CNAME source.
            - Only one custom domain is supported per storage account at this time.
            - To clear the existing custom domain, use an empty string for the custom domain name property.
            - Can be added to an existing storage account. Will be ignored during storage account creation.
        type: dict
        aliases:
            - custom_dns_domain_suffix
    kind:
        description:
            - The kind of storage.
            - The C(FileStorage) and (BlockBlobStorage) only used when I(account_type=Premium_LRS) or I(account_type=Premium_ZRS).
            - C(Storage) (general-purpose v1) and C(BlobStorage) are legacy account kinds that Azure is retiring. Microsoft
              is disabling creation of new legacy blob storage accounts in September 2026 and will fully retire both kinds
              in October 2026 (any remaining accounts will be auto-migrated to C(StorageV2)). Use C(StorageV2) for all new
              accounts.
        default: 'StorageV2'
        type: str
        choices:
            - Storage
            - StorageV2
            - BlobStorage
            - BlockBlobStorage
            - FileStorage
    is_hns_enabled:
        description:
            - Account HierarchicalNamespace enabled if sets to true.
            - When I(is_hns_enabled=True), I(kind) cannot be C(Storage).
        type: bool
    enable_nfs_v3:
        description:
            - NFS 3.0 protocol.
        type: bool
    access_tier:
        description:
            - The access tier for this storage account. Required when I(kind=BlobStorage).
        type: str
        choices:
            - Hot
            - Cool
            - Cold
            - Premium
            - Smart
    force_delete_nonempty:
        description:
            - Attempt deletion if resource already exists and cannot be updated.
        type: bool
        default: False
        aliases:
            - force
    https_only:
        description:
            - Allows https traffic only to storage service when set to C(True).
            - If omitted, new account creation will default to True, while existing accounts will not be change.
        type: bool
    minimum_tls_version:
        description:
            - The minimum required version of Transport Layer Security (TLS) for requests to a storage account.
            - If omitted, new account creation will default to null which is currently interpreted to TLS1_0. Existing accounts will not be modified.
        type: str
        choices:
            - TLS1_0
            - TLS1_1
            - TLS1_2
        version_added: "1.0.0"
    public_network_access:
        description:
            - Allow or disallow public network access to Storage Account.
        type: str
        choices:
            - Enabled
            - Disabled
        version_added: "1.12.0"
    allow_blob_public_access:
        description:
            - Allows blob containers in account to be set for anonymous public access.
            - If set to false, no containers in this account will be able to allow anonymous public access.
            - If omitted, new account creation will default to null which is currently interpreted to True. Existing accounts will not be modified.
        type: bool
        version_added: "1.1.0"
    network_acls:
        description:
            - Manages the Firewall and virtual networks settings of the storage account.
        type: dict
        suboptions:
            default_action:
                description:
                    - Default firewall traffic rule.
                    - If I(default_action=Allow) no other settings have effect.
                type: str
                choices:
                    - Allow
                    - Deny
                default: Allow
            bypass:
                description:
                    - When I(default_action=Deny) this controls which Azure components can still reach the Storage Account.
                    - The list is comma separated.
                    - It can be any combination of the example C(AzureServices), C(Logging), C(Metrics).
                    - If no Azure components are allowed, explicitly set I(bypass="").
                default: AzureServices
                type: str
            virtual_network_rules:
                description:
                    - A list of subnets and their actions.
                type: list
                elements: dict
                suboptions:
                    id:
                        description:
                            - The complete path to the subnet.
                        type: str
                        required: true
                    action:
                        description:
                            - The only logical I(action=Allow) because this setting is only accessible when I(default_action=Deny).
                        default: 'Allow'
                        type: str
                        choices:
                            - Allow
            ip_rules:
                description:
                    - A list of IP addresses or ranges in CIDR format.
                type: list
                elements: dict
                suboptions:
                    value:
                        description:
                            - The IP address or range.
                        type: str
                        required: true
                    action:
                        description:
                            - The only logical I(action=Allow) because this setting is only accessible when I(default_action=Deny).
                        default: 'Allow'
                        type: str
                        choices:
                            - Allow
            resource_access_rules:
                description:
                    - List of resource instance rules that allow specific Azure resources to access storage account when public network access is restricted.
                    - Supported resource types
                      U(https://learn.microsoft.com/en-us/azure/storage/common/storage-network-security-trusted-azure-services?source=recommendations)
                    - Resource instance rules only apply when I(default_action=Deny) and when I(public_network_access=Enabled).
                type: list
                elements: dict
                suboptions:
                    resource_id:
                        description:
                            - The full ARM resource ID of the Azure resource instance.
                        type: str
                        required: true
                    tenant_id:
                        description:
                            - The Azure Active Directory tenant ID of the resource instance.
                        type: str
                        required: true
    blob_cors:
        description:
            - Specifies CORS rules for the Blob service.
            - You can include up to five CorsRule elements in the request.
            - If no blob_cors elements are included in the argument list, nothing about CORS will be changed.
            - If you want to delete all CORS rules and disable CORS for the Blob service, explicitly set I(blob_cors=[]).
        type: list
        elements: dict
        suboptions:
            allowed_origins:
                description:
                    - A list of origin domains that will be allowed via CORS, or "*" to allow all domains.
                type: list
                elements: str
                required: true
            allowed_methods:
                description:
                    - A list of HTTP methods that are allowed to be executed by the origin.
                type: list
                elements: str
                required: true
            max_age_in_seconds:
                description:
                    - The number of seconds that the client/browser should cache a preflight response.
                type: int
                required: true
            exposed_headers:
                description:
                    - A list of response headers to expose to CORS clients.
                type: list
                elements: str
                required: true
            allowed_headers:
                description:
                    - A list of headers allowed to be part of the cross-origin request.
                type: list
                elements: str
                required: true
    static_website:
        description:
            - Manage static website configuration for the storage account.
        type: dict
        version_added: "1.13.0"
        suboptions:
            enabled:
                description:
                    - Indicates whether this account is hosting a static website.
                type: bool
                default: false
            index_document:
                description:
                    - The default name of the index page under each directory.
                type: str
            error_document404_path:
                description:
                    - The absolute path of the custom 404 page.
                type: str
    large_file_shares_state:
        description:
            - Allow large file shares if sets to Enabled.
        type: str
        choices:
            - Enabled
            - Disabled
    allow_cross_tenant_replication:
        description:
            - Allow or disallow cross AAD tenant object replication.
        type: bool
    allow_shared_key_access:
        description:
            - Indicates whether the storage account permits requests to be authorized with the account access key via Shared Key.
            - If false, then all requests, including shared access signatures, must be authorized with Azure Active Directory (Azure AD).
            - The default value is null, which is equivalent to true.
        type: bool
    default_to_o_auth_authentication:
        description:
            - A boolean flag which indicates whether the default authentication is OAuth or not.
            - The default interpretation is false for this property.
        type: bool
    smb_oauth_enabled:
        description:
            - Whether managed identities can access Azure Files SMB shares using OAuth.
            - When unset, the value on Azure is left unchanged (the service default is C(false) for new accounts).
        type: bool
    encryption:
        description:
            - The encryption settings on the storage account.
        type: dict
        suboptions:
            services:
                description:
                    -  List of services which support encryption.
                type: dict
                suboptions:
                    table:
                        description:
                            - The encryption function of the table storage service.
                        type: dict
                        suboptions:
                            enabled:
                                description:
                                    - Whether to encrypt the table type.
                                type: bool
                    queue:
                        description:
                            - The encryption function of the queue storage service.
                        type: dict
                        suboptions:
                            enabled:
                                description:
                                    - Whether to encrypt the queue type.
                                type: bool
                    file:
                        description:
                            - The encryption function of the file storage service.
                        type: dict
                        suboptions:
                            enabled:
                                description:
                                    - Whether to encrypt the file type.
                                type: bool
                    blob:
                        description:
                            - The encryption function of the blob storage service.
                        type: dict
                        suboptions:
                            enabled:
                                description:
                                    - Whether to encrypt the blob type.
                                type: bool
            key_source:
                description:
                    - The encryption keySource (provider).
                type: str
                default: Microsoft.Storage
                choices:
                    - Microsoft.Storage
                    - Microsoft.Keyvault
            key_vault_properties:
                description:
                  - list of Microsoft Keyvault properties needed in order to create Storage account with encryption enabled with Microsoft KeyVault for CMK.
                type: dict
                suboptions:
                  key_vault_uri:
                    description:
                        - The Uri of KeyVault.
                    type: str
                    required: true
                  key_name:
                    description:
                      - The name of KeyVault key.
                    type: str
                    required: true
                  key_version:
                    description:
                      - The version of KeyVault key.
                    type: str
            encryption_identity:
                description:
                    - The identity to be used with service-side encryption at rest.
                type: dict
                suboptions:
                  encryption_user_assigned_identity:
                    description:
                      - Resource identifier of the UserAssigned identity to be associated with server-side encryption on the storage account.
                    type: str
            require_infrastructure_encryption:
                description:
                    - A boolean indicating whether or not the service applies a secondary layer of encryption with platform managed keys for data at rest.
                type: bool
    immutable_storage_with_versioning:
        description:
            - The property is immutable and can only be set to true at the account creation time.
            - When set to true, it enables object level immutability for all the containers in the account by default.
        type: dict
        suboptions:
            enabled:
                description:
                    - A boolean flag which enables account-level immutability.
                    - All the containers under such an account have object-level immutability enabled by default.
                type: bool
            immutability_policy:
                description:
                    - Specifies the default account-level immutability policy which is inherited and
                      applied to objects that do not possess an explicit immutability policy at the object level.
                    - The object-level immutability policy has higher precedence than the container-level immutability policy,
                      which has a higher precedence than the account-level immutability policy.
                type: dict
                suboptions:
                    allow_protected_append_writes:
                        description:
                            - This property can only be changed for C(disabled) and C(unlocked) time-based retention policies.
                            - When C(enabled), new blocks can be written to an append blob while maintaining immutability protection and compliance.
                            - Only new blocks can be added and any existing blocks cannot be modified or deleted.
                        type: bool
                    state:
                        description:
                            - The ImmutabilityPolicy state defines the mode of the policy.
                        type: str
                        choices:
                            - Unlocked
                            - Locked
                            - Disabled
                    immutability_period_since_creation_in_days:
                        description:
                            - The immutability period for the blobs in the container since the policy creation, in days.
                        type: int

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
    - azure.azcollection.azure_identity_single

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)
'''

EXAMPLES = '''
- name: remove account, if it exists
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: clh0002
    state: absent

- name: create an account
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: clh0002
    type: Standard_RAGRS
    tags:
      testing: testing
      delete: on-exit

- name: Create an account with kind of FileStorage
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: c1h0002
    type: Premium_LRS
    kind: FileStorage
    tags:
      testing: testing

- name: Create storage account with I(enable_nfs_v3=false)
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: c1h0002
    account_type: Premium_LRS
    kind: FileStorage
    enable_nfs_v3: false
    static_website:
      enabled: true

- name: configure firewall and virtual networks
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: clh0002
    type: Standard_RAGRS
    network_acls:
      bypass: AzureServices,Metrics
      default_action: Deny
      virtual_network_rules:
        - id: /subscriptions/mySubscriptionId/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVnet/subnets/mySubnet
          action: Allow
      ip_rules:
        - value: 1.2.3.4
          action: Allow
        - value: 123.234.123.0/24
          action: Allow

- name: create an account with blob CORS
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: clh002
    type: Standard_RAGRS
    blob_cors:
      - allowed_origins:
          - http://www.example.com/
        allowed_methods:
          - GET
          - POST
        allowed_headers:
          - x-ms-meta-data*
          - x-ms-meta-target*
          - x-ms-meta-abc
        exposed_headers:
          - x-ms-meta-*
        max_age_in_seconds: 200

- name: Create new storage account with (key_source=Microsoft.Keyvault)
  azure.azcollection.azure_rm_storageaccount:
    resource_group: "{{ resource_group }}"
    name: "{{ storage_account_name }}"
    account_type: Standard_RAGRS
    kind: StorageV2
    identity:
      type: UserAssigned
      user_assigned_identity: "{{ managed_identity_ids[0] }}"
    encryption:
      services:
        blob:
          enabled: true
        file:
          enabled: true
      require_infrastructure_encryption: false
      key_source: Microsoft.Keyvault
      key_vault_properties:
        key_vault_uri: "https://vaultestfred001.vault.azure.net/"
        key_name: testkey
        key_version: 0bd2556671c64fc998095603878beb12
      encryption_identity:
        encryption_user_assigned_identity: "{{ managed_identity_ids[0] }}"

- name: Configure resource instance rules for Backup Vault and Synapse
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: mystorageacct
    account_type: Standard_LRS
    kind: StorageV2
    public_network_access: Enabled
    network_acls:
      default_action: Deny
      bypass: AzureServices
      resource_access_rules:
        - resource_id: "/subscriptions/mySubscriptionId/resourceGroups/myResourceGroup/providers/Microsoft.DataProtection/BackupVaults/myBackupVault"
          tenant_id: "myTenantId"
        - resource_id: "/subscriptions/mySubscriptionId/resourceGroups/myResourceGroup/providers/Microsoft.Synapse/workspaces/mySynapseWorkspace"
          tenant_id: "myTenantId"

- name: Remove all resource instance rules
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: mystorageacct
    network_acls:
      default_action: Deny
      bypass: AzureServices
      resource_access_rules: []

- name: Create a StorageV2 account with SMB OAuth enabled for Managed Identity access
  azure_rm_storageaccount:
    resource_group: myResourceGroup
    name: mystorageacct
    account_type: Standard_LRS
    kind: StorageV2
    smb_oauth_enabled: true
'''

RETURN = '''
state:
    description:
        - Current state of the storage account.
    returned: always
    type: complex
    contains:
        account_type:
            description:
                - Type of storage account.
            returned: always
            type: str
            sample: Standard_RAGRS
        allow_cross_tenant_replication:
            description:
                - Allow or disallow cross AAD tenant object replication.
            type: bool
            returned: always
            sample: true
        allow_shared_key_access:
            description:
                - Indicates whether the storage account permits requests to be authorized with the account access key via Shared Key.
            type: bool
            returned: always
            sample: true
        default_to_o_auth_authentication:
            description:
                - A boolean flag which indicates whether the default authentication is OAuth or not.
                - The default interpretation is false for this property.
            type: bool
            returned: always
            sample: true
        smb_oauth_enabled:
            description:
                - Whether managed identities can access Azure Files SMB shares using OAuth.
            type: bool
            returned: always
            sample: true
        custom_domain:
            description:
                - User domain assigned to the storage account.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - CNAME source.
                    returned: always
                    type: str
                    sample: testaccount
                use_sub_domain:
                    description:
                        - Whether to use sub domain.
                    returned: always
                    type: bool
                    sample: true
        encryption:
            description:
                - The encryption settings on the storage account.
            type: complex
            returned: always
            contains:
                key_source:
                    description:
                        - The encryption keySource (provider).
                    type: str
                    returned: always
                    sample: Microsoft.Storage
                require_infrastructure_encryption:
                    description:
                        - A boolean indicating whether or not the service applies a secondary layer of encryption with platform managed keys for data at rest.
                    type: bool
                    returned: always
                    sample: false
                services:
                    description:
                        - List of services which support encryption.
                    type: dict
                    returned: always
                    contains:
                        file:
                            description:
                                - The encryption function of the file storage service.
                            type: dict
                            returned: always
                            sample: {'enabled': true}
                        table:
                            description:
                                - The encryption function of the table storage service.
                            type: dict
                            returned: always
                            sample: {'enabled': true}
                        queue:
                            description:
                                - The encryption function of the queue storage service.
                            type: dict
                            returned: always
                            sample: {'enabled': true}
                        blob:
                            description:
                                - The encryption function of the blob storage service.
                            type: dict
                            returned: always
                            sample: {'enabled': true}
                encryption_identity:
                    description:
                        - The identity to be used with service-side encryption at rest.
                    type: dict
                    returned: always
                    sample: {"encryption_user_assigned_identity": \
                             "/subscriptions/xxxx/resourcegroups/testRG/providers/Microsoft.ManagedIdentity/userAssignedIdentities/identity-11"}
                key_vault_properties:
                    description:
                        - Dict of Microsoft Keyvault properties.
                        - Create the Storage account with encryption enabled with Microsoft KeyVault for CMK.
                    type: dict
                    returned: always
                    sample: {"key_name": "testkey",
                             "key_vault_uri": "https://vxxxxxxx01.vault.azure.net/",
                             "key_version": "0bd2556671c64fc998xxxxeb12"}
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/clh0003"
        is_hns_enabled:
            description:
                - Account HierarchicalNamespace enabled if sets to true.
            type: bool
            returned: always
            sample: true
        immutable_storage_with_versioning:
            description:
                - The property is immutable and can only be set to true at the account creation time.
                - When set to true, it enables object level immutability for all the containers in the account by default.
            type: complex
            returned: when-used
            contains:
                enabled:
                    description:
                        - A boolean flag which enables account-level immutability.
                        - All the containers under such an account have object-level immutability enabled by default.
                    type: bool
                    returned: when-used
                    sample: true
                immutability_policy:
                    description:
                        - Specifies the default account-level immutability policy which is inherited and
                          applied to objects that do not possess an explicit immutability policy at the object level.
                        - The object-level immutability policy has higher precedence than the container-level immutability policy,
                          which has a higher precedence than the account-level immutability policy.
                    type: dict
                    returned: when-used
                    contains:
                        allow_protected_append_writes:
                            description:
                                - This property can only be changed for disabled and unlocked time-based retention policies.
                                - When enabled, new blocks can be written to an append blob while maintaining immutability protection and compliance.
                                - Only new blocks can be added and any existing blocks cannot be modified or deleted.
                            type: bool
                            returned: when-used
                            sample: true
                        state:
                            description:
                                - The ImmutabilityPolicy state defines the mode of the policy.
                            type: str
                            returned: when-used
                            sample: Unlocked
                        immutability_period_since_creation_in_days:
                            description:
                                - The immutability period for the blobs in the container since the policy creation, in days.
                            type: int
                            returned: when-used
                            sample: true
        enable_nfs_v3:
            description:
                - NFS 3.0 protocol.
            type: bool
            returned: always
            sample: false
        location:
            description:
                - Valid Azure location. Defaults to location of the resource group.
            returned: always
            type: str
            sample: eastus2
        name:
            description:
                - Name of the storage account to update or create.
            returned: always
            type: str
            sample: clh0003
        network_acls:
            description:
                - A set of firewall and virtual network rules
            returned: always
            type: dict
            sample: {
                    "bypass": "AzureServices",
                    "default_action": "Deny",
                    "virtual_network_rules": [
                        {
                            "action": "Allow",
                            "id": "/subscriptions/mySubscriptionId/resourceGroups/myResourceGroup/ \
                                   providers/Microsoft.Network/virtualNetworks/myVnet/subnets/mySubnet"
                            }
                        ],
                    "ip_rules": [
                        {
                            "action": "Allow",
                            "value": "1.2.3.4"
                        },
                        {
                            "action": "Allow",
                            "value": "123.234.123.0/24"
                        }
                    ],
                    "resource_access_rules": [
                        {
                            "resource_id": "/subscriptions/mySubscriptionId/resourceGroups/myResourceGroup/ \
                                            providers/Microsoft.DataProtection/BackupVaults/myBackupVault",
                            "tenant_id": "myTenantId"
                        }
                    ]
                    }
        primary_endpoints:
            description:
                - The URLs to retrieve the public I(blob), I(queue), or I(table) object from the primary location.
            returned: always
            type: dict
            sample: {
                    "blob": "https://clh0003.blob.core.windows.net/",
                    "queue": "https://clh0003.queue.core.windows.net/",
                    "table": "https://clh0003.table.core.windows.net/"
                    }
        primary_location:
            description:
                - The location of the primary data center for the storage account.
            returned: always
            type: str
            sample: eastus2
        provisioning_state:
            description:
                - The status of the storage account.
                - Possible values include C(Creating), C(ResolvingDNS), C(Succeeded).
            returned: always
            type: str
            sample: Succeeded
        failover_in_progress:
            description:
                - Status indicating the storage account is currently failing over to its secondary location.
            returned: always
            type: bool
            sample: False
        resource_group:
            description:
                - The resource group's name.
            returned: always
            type: str
            sample: Testing
        secondary_endpoints:
            description:
                - The URLs to retrieve the public I(blob), I(queue), or I(table) object from the secondary location.
            returned: always
            type: dict
            sample: {
                    "blob": "https://clh0003-secondary.blob.core.windows.net/",
                    "queue": "https://clh0003-secondary.queue.core.windows.net/",
                    "table": "https://clh0003-secondary.table.core.windows.net/"
                    }
        secondary_location:
            description:
                - The location of the geo-replicated secondary for the storage account.
            returned: always
            type: str
            sample: centralus
        status_of_primary:
            description:
                - The status of the primary location of the storage account; either C(available) or C(unavailable).
            returned: always
            type: str
            sample: available
        status_of_secondary:
            description:
                - The status of the secondary location of the storage account; either C(available) or C(unavailable).
            returned: always
            type: str
            sample: available
        https_only:
            description:
                -  Allows https traffic only to storage service when set to C(true).
            returned: always
            type: bool
            sample: false
        minimum_tls_version:
            description:
                -  The minimum TLS version permitted on requests to storage.
            returned: always
            type: str
            sample: TLS1_2
        public_network_access:
            description:
                -  Public network access to Storage Account allowed or disallowed.
            returned: always
            type: str
            sample: Enabled
        allow_blob_public_access:
            description:
                -  Public access to all blobs or containers in the storage account allowed or disallowed.
            returned: always
            type: bool
            sample: true
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'tags1': 'value1' }
        type:
            description:
                - The storage account type.
            returned: always
            type: str
            sample: "Microsoft.Storage/storageAccounts"
        large_file_shares_state:
            description:
                - Allow large file shares if sets to Enabled.
            type: str
            returned: always
            sample: Enabled
        static_website:
            description:
                - Static website configuration for the storage account.
            returned: always
            version_added: "1.13.0"
            type: complex
            contains:
                enabled:
                    description:
                        - Whether this account is hosting a static website.
                    returned: always
                    type: bool
                    sample: true
                index_document:
                    description:
                        - The default name of the index page under each directory.
                    returned: always
                    type: str
                    sample: index.html
                error_document404_path:
                    description:
                        - The absolute path of the custom 404 page.
                    returned: always
                    type: str
                    sample: error.html
'''


import copy
import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AZURE_SUCCESS_STATE
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible.module_utils._text import to_native
from ansible.module_utils.basic import env_fallback

try:
    from azure.mgmt.storage.models import (Identity, UserAssignedIdentity)
    from azure.core.serialization import as_attribute_dict
except ImportError:
    # This is handled in azure_rm_common
    pass

cors_rule_spec = dict(
    allowed_origins=dict(type='list', elements='str', required=True),
    allowed_methods=dict(type='list', elements='str', required=True),
    max_age_in_seconds=dict(type='int', required=True),
    exposed_headers=dict(type='list', elements='str', required=True),
    allowed_headers=dict(type='list', elements='str', required=True),
)

static_website_spec = dict(
    enabled=dict(type='bool', default=False),
    index_document=dict(type='str'),
    error_document404_path=dict(type='str'),
)


file_spec = dict(
    enabled=dict(type='bool')
)


queue_spec = dict(
    enabled=dict(type='bool')
)


table_spec = dict(
    enabled=dict(type='bool')
)


blob_spec = dict(
    enabled=dict(type='bool')
)


def compare_cors(cors1, cors2):
    if len(cors1) != len(cors2):
        return False
    copy2 = copy.copy(cors2)
    for rule1 in cors1:
        matched = False
        for rule2 in copy2:
            if (rule1['max_age_in_seconds'] == rule2['max_age_in_seconds']
                    and set(rule1['allowed_methods']) == set(rule2['allowed_methods'])
                    and set(rule1['allowed_origins']) == set(rule2['allowed_origins'])
                    and set(rule1['allowed_headers']) == set(rule2['allowed_headers'])
                    and set(rule1['exposed_headers']) == set(rule2['exposed_headers'])):
                matched = True
                copy2.remove(rule2)
        if not matched:
            return False
    return True


class AzureRMStorageAccount(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            auth_mode=dict(
                type='str',
                choices=['key', 'login'],
                fallback=(env_fallback, ['AZURE_STORAGE_AUTH_MODE']),
                default="key"
            ),
            account_type=dict(type='str',
                              choices=['Premium_LRS', 'Standard_GRS', 'Standard_LRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_ZRS',
                                       'Standard_RAGZRS', 'Standard_GZRS'],
                              aliases=['type']),
            custom_domain=dict(type='dict', aliases=['custom_dns_domain_suffix']),
            location=dict(type='str'),
            name=dict(type='str', required=True),
            resource_group=dict(required=True, type='str', aliases=['resource_group_name']),
            state=dict(default='present', choices=['present', 'absent', 'failover']),
            force_delete_nonempty=dict(type='bool', default=False, aliases=['force']),
            tags=dict(type='dict'),
            kind=dict(type='str', default='StorageV2', choices=['Storage', 'StorageV2', 'BlobStorage', 'FileStorage', 'BlockBlobStorage']),
            access_tier=dict(type='str', choices=['Hot', 'Cool', 'Cold', 'Premium', 'Smart']),
            https_only=dict(type='bool'),
            minimum_tls_version=dict(type='str', choices=['TLS1_0', 'TLS1_1', 'TLS1_2']),
            public_network_access=dict(type='str', choices=['Enabled', 'Disabled']),
            allow_blob_public_access=dict(type='bool'),
            allow_shared_key_access=dict(type='bool'),
            allow_cross_tenant_replication=dict(type='bool'),
            default_to_o_auth_authentication=dict(type='bool'),
            smb_oauth_enabled=dict(type='bool'),
            network_acls=dict(
                type='dict',
                options=dict(
                    bypass=dict(type='str', default='AzureServices', no_log=False),
                    default_action=dict(type='str', choices=['Allow', 'Deny'], default='Allow'),
                    virtual_network_rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            id=dict(type='str', required=True),
                            action=dict(type='str', choices=['Allow'], default='Allow'),
                        ),
                    ),
                    ip_rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            value=dict(type='str', required=True),
                            action=dict(type='str', choices=['Allow'], default='Allow'),
                        ),
                    ),
                    resource_access_rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            resource_id=dict(type='str', required=True),
                            tenant_id=dict(type='str', required=True),
                        ),
                    ),
                ),
            ),
            blob_cors=dict(type='list', options=cors_rule_spec, elements='dict'),
            static_website=dict(type='dict', options=static_website_spec),
            is_hns_enabled=dict(type='bool'),
            large_file_shares_state=dict(type='str', choices=['Enabled', 'Disabled']),
            enable_nfs_v3=dict(type='bool'),
            encryption=dict(
                type='dict',
                options=dict(
                    services=dict(
                        type='dict',
                        options=dict(
                            blob=dict(
                                type='dict',
                                options=blob_spec
                            ),
                            table=dict(
                                type='dict',
                                options=table_spec
                            ),
                            queue=dict(
                                type='dict',
                                options=queue_spec
                            ),
                            file=dict(
                                type='dict',
                                options=file_spec
                            )
                        )
                    ),
                    require_infrastructure_encryption=dict(type='bool'),
                    key_source=dict(type='str', choices=["Microsoft.Storage", "Microsoft.Keyvault"], default='Microsoft.Storage'),
                    key_vault_properties=dict(
                        type='dict',
                        options=dict(
                            key_vault_uri=dict(type='str', required=True, no_log=True),
                            key_name=dict(type='str', required=True),
                            key_version=dict(type='str', no_log=True)
                        ), no_log=True
                    ),
                    encryption_identity=dict(
                        type='dict',
                        options=dict(
                            encryption_user_assigned_identity=dict(type='str', required=False)  # This will hold the resource ID of the user-assigned identity
                        )
                    )
                )
            ),
            identity=dict(
                type="dict",
                options=self.managed_identity_single_spec
            ),
            immutable_storage_with_versioning=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    immutability_policy=dict(
                        type='dict',
                        options=dict(
                            immutability_period_since_creation_in_days=dict(type='int'),
                            state=dict(type='str', choices=['Unlocked', 'Locked', 'Disabled']),
                            allow_protected_append_writes=dict(type='bool')
                        )
                    )
                )
            )
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.account_dict = None
        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.account_type = None
        self.custom_domain = None
        self.tags = None
        self.force_delete_nonempty = None
        self.kind = None
        self.access_tier = None
        self.https_only = None
        self.minimum_tls_version = None
        self.public_network_access = None
        self.allow_blob_public_access = None
        self.network_acls = None
        self.blob_cors = None
        self.static_website = None
        self.encryption = None
        self.is_hns_enabled = None
        self.large_file_shares_state = None
        self.enable_nfs_v3 = None
        self.allow_shared_key_access = None
        self.allow_cross_tenant_replication = None
        self.default_to_o_auth_authentication = None
        self.smb_oauth_enabled = None
        self._existing_azure_files_auth = None
        self._managed_identity = None
        self.identity = None
        self.update_identity = False
        self.immutable_storage_with_versioning = None

        super(AzureRMStorageAccount, self).__init__(self.module_arg_spec,
                                                    supports_check_mode=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {
                "identity": Identity,
                "user_assigned": UserAssignedIdentity,
            }
        return self._managed_identity

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        if len(self.name) < 3 or len(self.name) > 24:
            self.fail("Parameter error: name length must be between 3 and 24 characters.")

        if self.custom_domain:
            if self.custom_domain.get('name', None) is None:
                self.fail("Parameter error: expecting custom_domain to have a name attribute of type string.")
            if self.custom_domain.get('use_sub_domain', None) is None:
                self.fail("Parameter error: expecting custom_domain to have a use_sub_domain "
                          "attribute of type boolean.")

        if self.kind in ['FileStorage', 'BlockBlobStorage', ] and self.account_type not in ['Premium_LRS', 'Premium_ZRS']:
            self.fail("Parameter error: Storage account with {0} kind require account type is Premium_LRS or Premium_ZRS".format(self.kind))

        if self.state == 'present' and self.kind in ('Storage', 'BlobStorage'):
            self.module.deprecate(
                "kind='{0}' is a legacy Azure storage account type. Microsoft is disabling creation of new "
                "legacy blob storage accounts in September 2026 and fully retiring both 'Storage' "
                "(general-purpose v1) and 'BlobStorage' accounts in October 2026, after which remaining "
                "accounts will be auto-migrated to 'StorageV2'. Set kind='StorageV2' instead. See "
                "https://learn.microsoft.com/en-us/azure/storage/common/legacy-blob-storage-account-migration-overview "
                "and "
                "https://learn.microsoft.com/en-us/azure/storage/common/general-purpose-version-1-account-migration-overview".format(self.kind),
                version='4.0.0',
                collection_name='azure.azcollection',
            )
        self.account_dict = self.get_account()

        curr_identity = self.account_dict["identity"] if self.account_dict else None

        if self.identity:
            self.update_identity, identity_result = self.update_single_managed_identity(curr_identity=curr_identity,
                                                                                        new_identity=self.identity,
                                                                                        patch_support=True)
            self.identity = identity_result

        if self.state == 'present' and self.account_dict and \
           self.account_dict['provisioning_state'] != AZURE_SUCCESS_STATE:
            self.fail("Error: storage account {0} has not completed provisioning. State is {1}. Expecting state "
                      "to be {2}.".format(self.name, self.account_dict['provisioning_state'], AZURE_SUCCESS_STATE))

        if self.account_dict is not None:
            self.results['state'] = self.account_dict
        else:
            self.results['state'] = dict()

        if self.state == 'present':
            if not self.account_dict:
                self.results['state'] = self.create_account()
            else:
                self.results['state'] = self.update_account()
        elif self.state == 'absent' and self.account_dict:
            self.delete_account()
            self.results['state'] = dict(Status='Deleted')
        elif self.state == 'failover' and self.account_dict:
            self.failover_account()
            self.results['state'] = self.get_account()

        return self.results

    def check_name_availability(self):
        self.log('Checking name availability for {0}'.format(self.name))
        try:
            account_name = self.storage_models.StorageAccountCheckNameAvailabilityParameters(name=self.name)
            self.storage_client.storage_accounts.check_name_availability(account_name)
        except Exception as e:
            self.log('Error attempting to validate name.')
            self.fail("Error checking name availability: {0}".format(str(e)))

    def get_account(self):
        self.log('Get properties for account {0}'.format(self.name))
        account_obj = None
        blob_mgmt_props = None
        blob_client_props = None
        account_dict = None

        try:
            account_obj = self.storage_client.storage_accounts.get_properties(self.resource_group, self.name)
            blob_mgmt_props = self.storage_client.blob_services.get_service_properties(self.resource_group, self.name)
            if self.kind != "FileStorage":
                blob_client_props = self.get_blob_service_client(self.resource_group, self.name, self.auth_mode).get_service_properties()
        except Exception:
            pass

        if account_obj:
            account_dict = self.account_obj_to_dict(account_obj, blob_mgmt_props, blob_client_props)

        return account_dict

    def account_obj_to_dict(self, account_obj, blob_mgmt_props=None, blob_client_props=None):
        account_dict = dict(
            id=account_obj.id,
            name=account_obj.name,
            location=account_obj.location,
            failover_in_progress=(account_obj.failover_in_progress
                                  if account_obj.failover_in_progress is not None else False),
            resource_group=self.resource_group,
            type=account_obj.type,
            access_tier=account_obj.access_tier,
            sku_tier=account_obj.sku.tier,
            sku_name=account_obj.sku.name,
            provisioning_state=account_obj.provisioning_state,
            secondary_location=account_obj.secondary_location,
            status_of_primary=account_obj.status_of_primary,
            status_of_secondary=account_obj.status_of_secondary,
            primary_location=account_obj.primary_location,
            https_only=account_obj.enable_https_traffic_only,
            minimum_tls_version=account_obj.minimum_tls_version,
            public_network_access=account_obj.public_network_access,
            allow_blob_public_access=account_obj.allow_blob_public_access,
            default_to_o_auth_authentication=account_obj.default_to_o_auth_authentication,
            allow_cross_tenant_replication=account_obj.allow_cross_tenant_replication,
            allow_shared_key_access=account_obj.allow_shared_key_access,
            is_hns_enabled=account_obj.is_hns_enabled if account_obj.is_hns_enabled else False,
            enable_nfs_v3=account_obj.enable_nfs_v3 if hasattr(account_obj, 'enable_nfs_v3') else None,
            large_file_shares_state=account_obj.large_file_shares_state,
            static_website=dict(
                enabled=False,
                index_document=None,
                error_document404_path=None,
            ),
            immutable_storage_with_versioning=(
                as_attribute_dict(account_obj.immutable_storage_with_versioning)
                if account_obj.immutable_storage_with_versioning else None
            ),
        )
        account_dict['custom_domain'] = None
        if account_obj.custom_domain:
            account_dict['custom_domain'] = dict(
                name=account_obj.custom_domain.name,
                use_sub_domain=account_obj.custom_domain.use_sub_domain_name
            )

        account_dict['primary_endpoints'] = None
        if account_obj.primary_endpoints:
            account_dict['primary_endpoints'] = dict(
                blob=account_obj.primary_endpoints.blob,
                queue=account_obj.primary_endpoints.queue,
                table=account_obj.primary_endpoints.table
            )
        account_dict['secondary_endpoints'] = None
        if account_obj.secondary_endpoints:
            account_dict['secondary_endpoints'] = dict(
                blob=account_obj.secondary_endpoints.blob,
                queue=account_obj.secondary_endpoints.queue,
                table=account_obj.secondary_endpoints.table
            )
        account_dict['tags'] = None
        if account_obj.tags:
            account_dict['tags'] = account_obj.tags
        if blob_mgmt_props and blob_mgmt_props.cors and blob_mgmt_props.cors.cors_rules:
            account_dict['blob_cors'] = [dict(
                allowed_origins=[to_native(y) for y in x.allowed_origins],
                allowed_methods=[to_native(y) for y in x.allowed_methods],
                max_age_in_seconds=x.max_age_in_seconds,
                exposed_headers=[to_native(y) for y in x.exposed_headers],
                allowed_headers=[to_native(y) for y in x.allowed_headers]
            ) for x in blob_mgmt_props.cors.cors_rules]

        if blob_client_props and blob_client_props['static_website']:
            static_website = blob_client_props['static_website']
            account_dict['static_website'] = dict(
                enabled=static_website.enabled,
                index_document=static_website.index_document,
                error_document404_path=static_website.error_document404_path,
            )

        account_dict['network_acls'] = None
        if account_obj.network_rule_set:
            account_dict['network_acls'] = dict(
                bypass=account_obj.network_rule_set.bypass,
                default_action=account_obj.network_rule_set.default_action
            )
            account_dict['network_acls']['virtual_network_rules'] = []
            if account_obj.network_rule_set.virtual_network_rules:
                for rule in account_obj.network_rule_set.virtual_network_rules:
                    account_dict['network_acls']['virtual_network_rules'].append(dict(id=rule.virtual_network_resource_id, action=rule.action))

            account_dict['network_acls']['ip_rules'] = []
            if account_obj.network_rule_set.ip_rules:
                for rule in account_obj.network_rule_set.ip_rules:
                    account_dict['network_acls']['ip_rules'].append(dict(value=rule.ip_address_or_range, action=rule.action))

            account_dict['network_acls']['resource_access_rules'] = []
            if getattr(account_obj.network_rule_set, 'resource_access_rules', None):
                for rule in account_obj.network_rule_set.resource_access_rules:
                    account_dict['network_acls']['resource_access_rules'].append(dict(
                        resource_id=rule.resource_id,
                        tenant_id=rule.tenant_id
                    ))

        account_dict['encryption'] = as_attribute_dict(account_obj.encryption) if account_obj.encryption else dict()

        account_dict['identity'] = as_attribute_dict(account_obj.identity) if account_obj.identity else dict()

        afa = getattr(account_obj, 'azure_files_identity_based_authentication', None)
        smb = getattr(afa, 'smb_o_auth_settings', None) if afa else None
        account_dict['smb_oauth_enabled'] = bool(getattr(smb, 'is_smb_o_auth_enabled', False)) if smb else False
        self._existing_azure_files_auth = afa

        return account_dict

    def failover_account(self):
        if str(self.account_dict['sku_name']) not in ["Standard_GZRS", "Standard_GRS", "Standard_RAGZRS", "Standard_RAGRS"]:
            self.fail("Storage account SKU ({0}) does not support failover to a secondary region.".format(self.account_dict['sku_name']))
        try:
            account_obj = self.storage_client.storage_accounts.get_properties(self.resource_group, self.name, expand='georeplicationstats')
        except Exception as exc:
            self.fail("Error occurred while acquiring geo-replication status. {0}".format(str(exc)))

        if account_obj.failover_in_progress:
            self.fail("Storage account is already in process of failing over to secondary region.")

        if not account_obj.geo_replication_stats.can_failover:
            self.fail("Storage account is unable to failover.  Secondary region has status of {0}".format(account_obj.geo_replication_stats.status))

        try:
            poller = self.storage_client.storage_accounts.begin_failover(self.resource_group, self.name)
            result = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error occurred while attempting a failover operation. {0}".format(str(exc)))

        self.results['changed'] = True
        return result

    def update_network_rule_set(self):
        if not self.check_mode:
            try:
                models = self.storage_models
                nacls = self.network_acls or {}

                vnet_rules = None
                if nacls.get('virtual_network_rules'):
                    vnet_rules = [
                        models.VirtualNetworkRule(
                            virtual_network_resource_id=r.get('id'),
                            action=r.get('action', 'Allow')
                        ) for r in (nacls.get('virtual_network_rules') or [])
                    ]

                ip_rules = None
                if nacls.get('ip_rules'):
                    ip_rules = [
                        models.IPRule(
                            ip_address_or_range=r.get('value'),
                            action=r.get('action', 'Allow')
                        ) for r in (nacls.get('ip_rules') or [])
                    ]

                resource_access_rules = None
                # IMPORTANT: when desired list is [], send [] (not None) to clear rules
                if nacls.get('resource_access_rules') is not None:
                    resource_access_rules = [
                        models.ResourceAccessRule(
                            resource_id=r.get('resource_id'),
                            tenant_id=r.get('tenant_id')
                        ) for r in (nacls.get('resource_access_rules') or [])
                    ]
                nrs = models.NetworkRuleSet(
                    bypass=nacls.get('bypass'),
                    default_action=nacls.get('default_action', 'Allow'),
                    virtual_network_rules=vnet_rules,
                    ip_rules=ip_rules,
                    resource_access_rules=resource_access_rules
                )

                parameters = self.storage_models.StorageAccountUpdateParameters(network_rule_set=nrs)
                self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
            except Exception as exc:
                self.fail("Failed to update network ACLs: {0}".format(str(exc)))

    def update_account(self):
        self.log('Update storage account {0}'.format(self.name))
        if self.network_acls:
            if self.network_acls.get('default_action', 'Allow') != self.account_dict['network_acls']['default_action']:
                self.results['changed'] = True
                self.account_dict['network_acls']['default_action'] = self.network_acls['default_action']
                self.update_network_rule_set()

            if self.network_acls.get('default_action', 'Allow') == 'Deny':
                desired_rars = self.network_acls.get('resource_access_rules', None)
                current_rars = self.account_dict['network_acls'].get('resource_access_rules', [])
                if desired_rars is not None:
                    desired_ids = {r.get('resource_id') for r in (desired_rars or [])}
                    current_ids = {r.get('resource_id') for r in (current_rars or [])}

                    if desired_ids != current_ids:
                        self.results['changed'] = True
                        # Keep local state in sync for readability
                        self.account_dict['network_acls']['resource_access_rules'] = desired_rars
                        self.update_network_rule_set()

                if self.network_acls.get('bypass') != self.account_dict['network_acls'].get('bypass'):
                    self.results['changed'] = True
                    self.account_dict['network_acls']['bypass'] = self.network_acls['bypass']
                    self.update_network_rule_set()

                if self.network_acls.get('virtual_network_rules') is not None:
                    desired_vnet_ids = {r.get('id') for r in (self.network_acls.get('virtual_network_rules') or [])}
                    current_vnet_ids = {r.get('id') for r in (self.account_dict['network_acls'].get('virtual_network_rules') or [])}

                    if desired_vnet_ids != current_vnet_ids:
                        self.results['changed'] = True
                        self.account_dict['network_acls']['virtual_network_rules'] = self.network_acls.get('virtual_network_rules') or []
                        self.update_network_rule_set()

                if self.network_acls.get('ip_rules') is not None:
                    desired_ip_values = {r.get('value') for r in (self.network_acls.get('ip_rules') or [])}
                    current_ip_values = {r.get('value') for r in (self.account_dict['network_acls'].get('ip_rules') or [])}

                    if desired_ip_values != current_ip_values:
                        self.results['changed'] = True
                        self.account_dict['network_acls']['ip_rules'] = self.network_acls.get('ip_rules') or []
                        self.update_network_rule_set()

        if self.enable_nfs_v3 is not None and bool(self.enable_nfs_v3) != bool(self.account_dict.get('enable_nfs_v3')):
            self.results['changed'] = True
            self.account_dict['enable_nfs_v3'] = self.enable_nfs_v3

        if self.is_hns_enabled is not None and bool(self.is_hns_enabled) != bool(self.account_dict.get('is_hns_enabled')):
            self.results['changed'] = True
            self.account_dict['is_hns_enabled'] = self.is_hns_enabled
            if not self.check_mode:
                self.fail("The is_hns_enabled parameter not support to update, from {0} to {1}".
                          format(bool(self.account_dict.get('is_hns_enabled')), self.is_hns_enabled))

        if self.https_only is not None and bool(self.https_only) != bool(self.account_dict.get('https_only')):
            self.results['changed'] = True
            self.account_dict['https_only'] = self.https_only
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(enable_https_traffic_only=self.https_only)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update https only: {0}".format(str(exc)))

        if self.minimum_tls_version is not None and self.minimum_tls_version != self.account_dict.get('minimum_tls_version'):
            self.results['changed'] = True
            self.account_dict['minimum_tls_version'] = self.minimum_tls_version
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(minimum_tls_version=self.minimum_tls_version)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update minimum tls: {0}".format(str(exc)))

        if self.public_network_access is not None and self.public_network_access != self.account_dict.get('public_network_access'):
            self.results['changed'] = True
            self.account_dict['public_network_access'] = self.public_network_access
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(public_network_access=self.public_network_access)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update public network access: {0}".format(str(exc)))

        if self.allow_blob_public_access is not None and self.allow_blob_public_access != self.account_dict.get('allow_blob_public_access'):
            self.results['changed'] = True
            self.account_dict['allow_blob_public_access'] = self.allow_blob_public_access
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(allow_blob_public_access=self.allow_blob_public_access)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update allow public blob access: {0}".format(str(exc)))

        if self.allow_shared_key_access is not None and self.allow_shared_key_access != self.account_dict.get('allow_shared_key_access'):
            self.results['changed'] = True
            self.account_dict['allow_shared_key_access'] = self.allow_shared_key_access
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(allow_shared_key_access=self.allow_shared_key_access)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update allow shared key access: {0}".format(str(exc)))

        if self.allow_cross_tenant_replication is not None and self.allow_cross_tenant_replication != self.account_dict.get('allow_cross_tenant_replication'):
            self.results['changed'] = True
            self.account_dict['allow_cross_tenant_replication'] = self.allow_cross_tenant_replication
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(allow_cross_tenant_replication=self.allow_cross_tenant_replication)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update allow cross tenant replication: {0}".format(str(exc)))

        if self.default_to_o_auth_authentication is not None and \
           self.default_to_o_auth_authentication != self.account_dict.get('default_to_o_auth_authentication'):
            self.results['changed'] = True
            self.account_dict['default_to_o_auth_authentication'] = self.default_to_o_auth_authentication
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(default_to_o_auth_authentication=self.default_to_o_auth_authentication)
                    self.storage_client.storage_accounts.update(self.resource_group,
                                                                self.name,
                                                                parameters)
                except Exception as exc:
                    self.fail("Failed to update default_to_o_auth_authentication: {0}".format(str(exc)))

        if self.account_type:
            if self.account_type != self.account_dict['sku_name']:
                # change the account type
                SkuName = self.storage_models.SkuName
                if self.account_dict['sku_name'] in [SkuName.premium_lrs, SkuName.standard_zrs]:
                    self.fail("Storage accounts of type {0} and {1} cannot be changed.".format(
                        SkuName.premium_lrs, SkuName.standard_zrs))
                if self.account_type in [SkuName.premium_lrs, SkuName.standard_zrs]:
                    self.fail("Storage account of type {0} cannot be changed to a type of {1} or {2}.".format(
                        self.account_dict['sku_name'], SkuName.premium_lrs, SkuName.standard_zrs))

                self.results['changed'] = True
                self.account_dict['sku_name'] = self.account_type

                if self.results['changed'] and not self.check_mode:
                    # Perform the update. The API only allows changing one attribute per call.
                    try:
                        self.log("sku_name: %s" % self.account_dict['sku_name'])
                        self.log("sku_tier: %s" % self.account_dict['sku_tier'])
                        sku = self.storage_models.Sku(name=SkuName(self.account_dict['sku_name']))
                        sku.tier = self.storage_models.SkuTier(self.account_dict['sku_tier'])
                        parameters = self.storage_models.StorageAccountUpdateParameters(sku=sku)
                        self.storage_client.storage_accounts.update(self.resource_group,
                                                                    self.name,
                                                                    parameters)
                    except Exception as exc:
                        self.fail("Failed to update account type: {0}".format(str(exc)))

        if self.custom_domain:
            if not self.account_dict['custom_domain'] or self.account_dict['custom_domain'] != self.custom_domain:
                self.results['changed'] = True
                self.account_dict['custom_domain'] = self.custom_domain

            if self.results['changed'] and not self.check_mode:
                new_domain = self.storage_models.CustomDomain(name=self.custom_domain['name'],
                                                              use_sub_domain_name=self.custom_domain['use_sub_domain'])
                parameters = self.storage_models.StorageAccountUpdateParameters(custom_domain=new_domain)
                try:
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update custom domain: {0}".format(str(exc)))

        if self.update_identity:
            self.results['changed'] = True
            parameters = self.storage_models.StorageAccountUpdateParameters(identity=self.identity)
            try:
                self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
            except Exception as exc:
                self.fail("Failed to update access tier: {0}".format(str(exc)))

        if self.access_tier:
            if not self.account_dict['access_tier'] or self.account_dict['access_tier'] != self.access_tier:
                self.results['changed'] = True
                self.account_dict['access_tier'] = self.access_tier

            if self.results['changed'] and not self.check_mode:
                parameters = self.storage_models.StorageAccountUpdateParameters(access_tier=self.access_tier)
                try:
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update access tier: {0}".format(str(exc)))

        if self.large_file_shares_state is not None:
            if self.large_file_shares_state != self.account_dict['large_file_shares_state']:
                self.results['changed'] = True
                self.account_dict['large_file_shares_state'] = self.large_file_shares_state

            if self.results['changed'] and not self.check_mode:
                if self.large_file_shares_state == 'Disabled':
                    parameters = self.storage_models.StorageAccountUpdateParameters(large_file_shares_state=None)
                else:
                    parameters = self.storage_models.StorageAccountUpdateParameters(large_file_shares_state=self.large_file_shares_state)
                try:
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update large_file_shares_state: {0}".format(str(exc)))

        if self.smb_oauth_enabled is not None and \
           self.smb_oauth_enabled != self.account_dict.get('smb_oauth_enabled'):
            self.results['changed'] = True
            self.account_dict['smb_oauth_enabled'] = self.smb_oauth_enabled
            if not self.check_mode:
                try:
                    parameters = self.storage_models.StorageAccountUpdateParameters(
                        azure_files_identity_based_authentication=self._build_azure_files_identity_auth_payload(
                            existing_afa=self._existing_azure_files_auth))
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update smb_oauth_enabled: {0}".format(str(exc)))

        update_tags, self.account_dict['tags'] = self.update_tags(self.account_dict['tags'])
        if update_tags:
            self.results['changed'] = True
            if not self.check_mode:
                parameters = self.storage_models.StorageAccountUpdateParameters(tags=self.account_dict['tags'])
                try:
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update tags: {0}".format(str(exc)))

        if not self.default_compare({}, self.immutable_storage_with_versioning, self.account_dict['immutable_storage_with_versioning'], '', dict(compare=[])):
            self.results['changed'] = True
            if not self.check_mode:
                parameters = self.storage_models.StorageAccountUpdateParameters(
                    immutable_storage_with_versioning=self._build_immutable_storage_payload())
                try:
                    self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
                except Exception as exc:
                    self.fail("Failed to update immutable_storage_with_versioning: {0}".format(str(exc)))

        if self.blob_cors and not compare_cors(self.account_dict.get('blob_cors', []), self.blob_cors):
            self.results['changed'] = True
            if not self.check_mode:
                self.set_blob_cors()

        if self.static_website and self.static_website != self.account_dict.get("static_website", dict()):
            self.results['changed'] = True
            self.account_dict['static_website'] = self.static_website
            self.update_static_website()

        if self.encryption is not None:
            encryption_changed = False
            if self.encryption.get('require_infrastructure_encryption') and bool(self.encryption.get('require_infrastructure_encryption')) \
                    != bool(self.account_dict['encryption']['require_infrastructure_encryption']):
                encryption_changed = True

            if self.encryption.get('key_source') != self.account_dict['encryption']['key_source']:
                encryption_changed = True

            if self.encryption.get('services') is not None:
                if self.encryption.get('queue') is not None and self.account_dict['encryption']['services'].get('queue') is not None:
                    encryption_changed = True
                if self.encryption.get('file') is not None and self.account_dict['encryption']['services'].get('file') is not None:
                    encryption_changed = True
                if self.encryption.get('table') is not None and self.account_dict['encryption']['services'].get('table') is not None:
                    encryption_changed = True
                if self.encryption.get('blob') is not None and self.account_dict['encryption']['services'].get('blob') is not None:
                    encryption_changed = True
            if not self.default_compare({}, self.encryption.get('key_vault_properties'),
                                        self.account_dict['encryption'].get('key_vault_properties'), '', dict(compare=[])):
                encryption_changed = True
            if not self.default_compare({}, self.encryption.get('encryption_identity'),
                                        self.account_dict['encryption'].get('encryption_identity'), '', dict(compare=[])):
                encryption_changed = True

            if encryption_changed and not self.check_mode:
                self.fail("The encryption can't update encryption, encryption info as {0}".format(self.account_dict['encryption']))
        time.sleep(1)
        return self.get_account()

    def _build_encryption_payload(self):
        '''
        Build an Encryption SDK model with all nested fields explicitly typed.
        '''
        if self.encryption is None:
            return None
        services = self.encryption.get('services') or {}
        kvp = self.encryption.get('key_vault_properties')
        ei = self.encryption.get('encryption_identity')
        return self.storage_models.Encryption(
            key_source=self.encryption.get('key_source'),
            require_infrastructure_encryption=self.encryption.get('require_infrastructure_encryption'),
            services=self.storage_models.EncryptionServices(
                blob=self.storage_models.EncryptionService(**services['blob']) if services.get('blob') else None,
                file=self.storage_models.EncryptionService(**services['file']) if services.get('file') else None,
                queue=self.storage_models.EncryptionService(**services['queue']) if services.get('queue') else None,
                table=self.storage_models.EncryptionService(**services['table']) if services.get('table') else None,
            ) if services else None,
            key_vault_properties=self.storage_models.KeyVaultProperties(**kvp) if kvp else None,
            encryption_identity=self.storage_models.EncryptionIdentity(**ei) if ei else None,
        )

    def _build_immutable_storage_payload(self):
        '''
        Build an ImmutableStorageAccount SDK model with the nested policy typed.
        '''
        if self.immutable_storage_with_versioning is None:
            return None
        policy = self.immutable_storage_with_versioning.get('immutability_policy')
        return self.storage_models.ImmutableStorageAccount(
            enabled=self.immutable_storage_with_versioning.get('enabled'),
            immutability_policy=self.storage_models.AccountImmutabilityPolicyProperties(**policy) if policy else None,
        )

    def _build_azure_files_identity_auth_payload(self, existing_afa=None):
        '''
        Build an AzureFilesIdentityBasedAuthentication SDK model that carries the
        smb_oauth_enabled toggle. Returns None when the caller did not set the
        parameter.
        '''
        if self.smb_oauth_enabled is None:
            return None

        smb_settings = self.storage_models.SmbOAuthSettings(
            is_smb_o_auth_enabled=self.smb_oauth_enabled,
        )

        directory_service_options = getattr(existing_afa, 'directory_service_options', None) or "None"
        active_directory_properties = getattr(existing_afa, 'active_directory_properties', None)
        default_share_permission = getattr(existing_afa, 'default_share_permission', None)

        return self.storage_models.AzureFilesIdentityBasedAuthentication(
            directory_service_options=directory_service_options,
            active_directory_properties=active_directory_properties,
            default_share_permission=default_share_permission,
            smb_o_auth_settings=smb_settings,
        )

    def create_account(self):
        self.log("Creating account {0}".format(self.name))

        if not self.location:
            self.fail('Parameter error: location required when creating a storage account.')

        if not self.account_type:
            self.fail('Parameter error: account_type required when creating a storage account.')

        if not self.access_tier and self.kind == 'BlobStorage':
            self.fail('Parameter error: access_tier required when creating a storage account of type BlobStorage.')

        self.check_name_availability()
        self.results['changed'] = True

        if self.check_mode:
            account_dict = dict(
                location=self.location,
                account_type=self.account_type,
                name=self.name,
                resource_group=self.resource_group,
                enable_https_traffic_only=self.https_only,
                minimum_tls_version=self.minimum_tls_version,
                public_network_access=self.public_network_access,
                allow_blob_public_access=self.allow_blob_public_access,
                encryption=self.encryption,
                is_hns_enabled=self.is_hns_enabled,
                enable_nfs_v3=self.enable_nfs_v3,
                large_file_shares_state=self.large_file_shares_state,
                default_to_o_auth_authentication=self.default_to_o_auth_authentication,
                allow_cross_tenant_replication=self.allow_cross_tenant_replication,
                allow_shared_key_access=self.allow_shared_key_access,
                identity=as_attribute_dict(self.identity) if self.identity else None,
                immutable_storage_with_versioning=self.immutable_storage_with_versioning,
                smb_oauth_enabled=self.smb_oauth_enabled,
                tags=dict()
            )
            if self.tags:
                account_dict['tags'] = self.tags
            if self.network_acls:
                account_dict['network_acls'] = self.network_acls
            if self.blob_cors:
                account_dict['blob_cors'] = self.blob_cors
            if self.static_website:
                account_dict['static_website'] = self.static_website
            return account_dict
        sku = self.storage_models.Sku(name=self.storage_models.SkuName(self.account_type))
        sku.tier = self.storage_models.SkuTier.standard if 'Standard' in self.account_type else \
            self.storage_models.SkuTier.premium
        # pylint: disable=missing-kwoa
        azure_files_auth = self._build_azure_files_identity_auth_payload()
        parameters = self.storage_models.StorageAccountCreateParameters(sku=sku,
                                                                        kind=self.kind,
                                                                        location=self.location,
                                                                        tags=self.tags,
                                                                        identity=self.identity,
                                                                        enable_https_traffic_only=self.https_only,
                                                                        minimum_tls_version=self.minimum_tls_version,
                                                                        public_network_access=self.public_network_access,
                                                                        allow_blob_public_access=self.allow_blob_public_access,
                                                                        encryption=self._build_encryption_payload(),
                                                                        is_hns_enabled=self.is_hns_enabled,
                                                                        enable_nfs_v3=self.enable_nfs_v3,
                                                                        access_tier=self.access_tier,
                                                                        allow_shared_key_access=self.allow_shared_key_access,
                                                                        default_to_o_auth_authentication=self.default_to_o_auth_authentication,
                                                                        allow_cross_tenant_replication=self.allow_cross_tenant_replication,
                                                                        immutable_storage_with_versioning=self._build_immutable_storage_payload(),
                                                                        azure_files_identity_based_authentication=azure_files_auth,
                                                                        large_file_shares_state=self.large_file_shares_state)
        self.log(str(parameters))
        try:
            poller = self.storage_client.storage_accounts.begin_create(self.resource_group, self.name, parameters)
            self.get_poller_result(poller)
        except Exception as e:
            self.log('Error creating storage account.')
            self.fail("Failed to create account: {0}".format(str(e)))
        if self.network_acls:
            self.set_network_acls()
        if self.blob_cors:
            self.set_blob_cors()
        if self.static_website:
            self.update_static_website()
        return self.get_account()

    def delete_account(self):
        if self.account_dict['provisioning_state'] == self.storage_models.ProvisioningState.succeeded.value and \
           not self.force_delete_nonempty and self.account_has_blob_containers():
            self.fail("Account contains blob containers. Is it in use? Use the force_delete_nonempty option to attempt deletion.")

        self.log('Delete storage account {0}'.format(self.name))
        self.results['changed'] = True
        if not self.check_mode:
            try:
                status = self.storage_client.storage_accounts.delete(self.resource_group, self.name)
                self.log("delete status: ")
                self.log(str(status))
            except Exception as e:
                self.fail("Failed to delete the account: {0}".format(str(e)))
        return True

    def account_has_blob_containers(self):
        '''
        If there are blob containers, then there are likely VMs depending on this account and it should
        not be deleted.
        '''
        if self.kind == "FileStorage":
            return False
        self.log('Checking for existing blob containers')
        blob_service = self.get_blob_service_client(self.resource_group, self.name, self.auth_mode)
        try:
            response = blob_service.list_containers()
        except Exception:
            # No blob storage available?
            return False

        if len(list(response)) > 0:
            return True
        return False

    def set_blob_cors(self):
        try:
            cors_rules = self.storage_models.CorsRules(cors_rules=[self.storage_models.CorsRule(**x) for x in self.blob_cors])
            self.storage_client.blob_services.set_service_properties(self.resource_group,
                                                                     self.name,
                                                                     self.storage_models.BlobServiceProperties(cors=cors_rules))
        except Exception as exc:
            self.fail("Failed to set CORS rules: {0}".format(str(exc)))

    def update_static_website(self):
        if self.kind == "FileStorage":
            return
        try:
            self.get_blob_service_client(self.resource_group, self.name, self.auth_mode).set_service_properties(static_website=self.static_website)
        except Exception as exc:
            self.fail("Failed to set static website config: {0}".format(str(exc)))

    def set_network_acls(self):
        try:
            models = self.storage_models
            nacls = self.network_acls or {}

            def build_network_rule_set(nacls_local):
                vnet_rules = None
                if nacls_local.get('virtual_network_rules'):
                    vnet_rules = [
                        models.VirtualNetworkRule(
                            virtual_network_resource_id=r.get('id'),
                            action=r.get('action', 'Allow')
                        ) for r in (nacls_local.get('virtual_network_rules') or [])
                    ]

                ip_rules = None
                if nacls_local.get('ip_rules'):
                    ip_rules = [
                        models.IPRule(
                            ip_address_or_range=r.get('value'),
                            action=r.get('action', 'Allow')
                        ) for r in (nacls_local.get('ip_rules') or [])
                    ]

                resource_access_rules = None
                if nacls_local.get('resource_access_rules'):
                    resource_access_rules = [
                        models.ResourceAccessRule(
                            resource_id=r.get('resource_id'),
                            tenant_id=r.get('tenant_id')
                        ) for r in (nacls_local.get('resource_access_rules') or [])
                    ]

                return models.NetworkRuleSet(
                    bypass=nacls_local.get('bypass'),
                    default_action=nacls_local.get('default_action', 'Allow'),
                    virtual_network_rules=vnet_rules,
                    ip_rules=ip_rules,
                    resource_access_rules=resource_access_rules
                )

            nrs = build_network_rule_set(nacls)
            parameters = self.storage_models.StorageAccountUpdateParameters(network_rule_set=nrs)
            self.storage_client.storage_accounts.update(self.resource_group, self.name, parameters)
        except Exception as exc:
            self.fail("Failed to update network ACLs: {0}".format(str(exc)))


def main():
    AzureRMStorageAccount()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_cellular_data_profiles
short_description: Resource module for organizations _devices _cellular _data _profiles
description:
  - Manage operations create, update and delete of the resource organizations _devices
    _cellular _data _profiles. - > Add a cellular data management profile to this
    organization. Creates a cellular data management profile in this organization
    and returns the created profile, including its rules and actions. - > Delete a
    cellular data management profile from this organization. Removes the profile,
    including its associated rules and node assignments, and notifies affected devices
    of the resulting configuration change. - > Update a Cellular Data Management Profile.
    Note that changes made to this endpoint will overwrite existing settings for the
    profile so the entire profile, rules and actions should be sent when making an
    update.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  description:
    description: Description of the profile to be added.
    type: str
  name:
    description: Name of the profile to be added. This must be unique.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  profileId:
    description: ID of the profile.
    type: str
  rules:
    description: The rules associated with this profile. At least one rule and no
      more than two rules may be defined for a profile.
    elements: dict
    suboptions:
      actions:
        description: The defined actions that will occur when the profile rule is
          triggered. No more than two actions may be defined for a rule.
        elements: dict
        suboptions:
          type:
            description: The type of action to be undertaken. One of 'send_message'
              or 'failover'.
            type: str
        type: list
      cap:
        description: The data cap values to be set with this rule.
        suboptions:
          term:
            description: The data usage term configuration that determines when the
              aggregated data count resets on the device.
            suboptions:
              resets:
                description: How often the device resets the aggregated data count.
                  One of 'daily', 'weekly', or 'monthly'.
                type: str
              starts:
                description: The reset point for the selected term. Exactly one of
                  the nested values must be set depending on the selected reset frequency.
                  Use 'hourOfDay' for a daily reset, 'dayOfWeek' for a weekly reset,
                  and 'dayOfMonth' for a monthly reset.
                suboptions:
                  dayOfMonth:
                    description: When the data usage term is to be reset monthly,
                      the day of month that the accounted bandwidth usage for the
                      term should be reset. This can be between 1 and 31 (months that
                      have a number of days less than the value for dayOfMonth will
                      begin on the last day of the month).
                    type: int
                  dayOfWeek:
                    description: When the data usage term resets weekly, the day of
                      week that the accounted bandwidth usage should reset. Uses the
                      lowercase three-letter weekday abbreviation 'mon', 'tue', 'wed',
                      'thu', 'fri', 'sat', or 'sun'. Requests are normalized case-insensitively
                      to this form.
                    type: str
                  hourOfDay:
                    description: When the data usage term is to be reset daily, the
                      hour the accounted bandwidth usage for the term should be reset.
                      This can be between 0 and 23 (where 0 is 0 00 GMT and 23 is
                      23 00 GMT).
                    type: int
                type: dict
            type: dict
          threshold:
            description: The data usage threshold at which the rule should be triggered.
              This is expressed as a float between 0.01 and 1.0 (where 0.01 is equal
              to 1% of total and 1.0 is equal to 100%). By default this will be 1.0.
            type: float
          value:
            description: The total bandwidth available for the specified term in megabytes.
            type: int
        type: dict
      slot:
        description: The SIM slot that the rule is applied to. One of 'sim1', 'sim2',
          or 'sim3'. Devices included in this group must have an active card/profile
          with the defined SIM slot in order to use this rule.
        type: str
      uplink:
        description: Configuration for the uplink governed by this rule.
        suboptions:
          isPreferred:
            description: Whether this uplink is the preferred one to use once all
              rules have been exhausted.
            type: bool
          priority:
            description: The uplink priority for this rule. One of 1 or 2, where 1
              is the highest priority.
            type: int
        type: dict
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationDevicesCellularDataProfile
    description: Complete reference of the createOrganizationDevicesCellularDataProfile
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-devices-cellular-data-profile
  - name: Cisco Meraki documentation for organizations deleteOrganizationDevicesCellularDataProfile
    description: Complete reference of the deleteOrganizationDevicesCellularDataProfile
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-devices-cellular-data-profile
  - name: Cisco Meraki documentation for organizations updateOrganizationDevicesCellularDataProfile
    description: Complete reference of the updateOrganizationDevicesCellularDataProfile
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-devices-cellular-data-profile
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_devices_cellular_data_profile,
    organizations.Organizations.delete_organization_devices_cellular_data_profile,
    organizations.Organizations.update_organization_devices_cellular_data_profile,
  - Paths used are
    post /organizations/{organizationId}/devices/cellular/data/profiles,
    delete /organizations/{organizationId}/devices/cellular/data/profiles/{profileId},
    put /organizations/{organizationId}/devices/cellular/data/profiles/{profileId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_devices_cellular_data_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    description: some description
    name: some unique profile name
    organizationId: string
    rules:
      - actions:
          - type: failover
        cap:
          term:
            resets: daily
            starts:
              dayOfMonth: 2
              dayOfWeek: mon
              hourOfDay: 12
          threshold: 0.9
          value: 42
        slot: sim1
        uplink:
          isPreferred: true
          priority: 1
- name: Update by id
  cisco.meraki.organizations_devices_cellular_data_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    description: some description
    organizationId: string
    profileId: '42'
    rules:
      - actions:
          - type: failover
        cap:
          term:
            resets: daily
            starts:
              dayOfMonth: 2
              dayOfWeek: mon
              hourOfDay: 12
          threshold: 0.9
          value: 42
        slot: sim1
        uplink:
          isPreferred: true
          priority: 1
- name: Delete by id
  cisco.meraki.organizations_devices_cellular_data_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    organizationId: string
    profileId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "profileId": "string",
      "lastUpdatedAt": "string",
      "name": "string",
      "description": "string",
      "rules": [
        {
          "ruleId": "string",
          "slot": "string",
          "uplink": {
            "priority": 0,
            "isPreferred": true
          },
          "cap": {
            "value": 0,
            "threshold": 0,
            "term": {
              "resets": "string",
              "starts": {
                "hourOfDay": 0,
                "dayOfWeek": "string",
                "dayOfMonth": 0
              }
            }
          },
          "actions": [
            {
              "type": "string"
            }
          ]
        }
      ]
    }
"""



# Copyright (c) 2026, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

# Maps state values across all kopia modules to their kopia CLI subcommands.
# Used with cmd_runner_fmt.as_map() for the 'state' arg format.
STATE_MAP = {
    # kopia_repository
    "created": "create",
    "connected": "connect",
    "disconnected": "disconnect",
    "synced": "sync-to",
    "throttled": "throttle",
}

# Maps backend provider names to their CLI parameter definitions.
# Each provider maps param_name -> (flag, type) where type is:
#   "str"  - emit --flag=value (skip if None)
#   "bool" - emit --flag only when value is True (skip if False or None)
#   "list" - emit --flag item once per item in the list (skip if empty or None)
# The "server" provider is intentionally absent: `kopia repository connect server`
# uses top-level flags (--url, --server-cert-fingerprint) rather than backend flags,
# so fmt_backend() returns [] for it and those flags are passed separately.
_PROVIDER_BACKEND_MAP: dict[str, dict[str, tuple[str, str]]] = {
    "azure": {
        "container": ("--container", "str"),
        "storage_account": ("--storage-account", "str"),
        "storage_key": ("--storage-key", "str"),
        "sas_token": ("--sas-token", "str"),
        "storage_domain": ("--storage-domain", "str"),
        "prefix": ("--prefix", "str"),
        "client_id": ("--client-id", "str"),
        "client_secret": ("--client-secret", "str"),
        "tenant_id": ("--tenant-id", "str"),
        "azure_federated_token_file": ("--azure-federated-token-file", "str"),
    },
    "b2": {
        "bucket": ("--bucket", "str"),
        "access_key": ("--key-id", "str"),
        "secret_access_key": ("--key", "str"),
        "prefix": ("--prefix", "str"),
    },
    "filesystem": {
        "path": ("--path", "str"),
    },
    "gcs": {
        "bucket": ("--bucket", "str"),
        "credentials_file": ("--credentials-file", "str"),
        "prefix": ("--prefix", "str"),
        "embed_credentials": ("--embed-credentials", "bool"),
        "read_only": ("--read-only", "bool"),
    },
    "gdrive": {
        "folder_id": ("--folder-id", "str"),
        "credentials_file": ("--credentials-file", "str"),
        "read_only": ("--read-only", "bool"),
    },
    "rclone": {
        "path": ("--remote-path", "str"),
        "rclone_exe": ("--rclone-exe", "str"),
        "rclone_args": ("--rclone-args", "list"),
        "rclone_env": ("--rclone-env", "list"),
        "embed_rclone_config": ("--embed-rclone-config", "bool"),
    },
    "s3": {
        "bucket": ("--bucket", "str"),
        "access_key": ("--access-key", "str"),
        "secret_access_key": ("--secret-access-key", "str"),
        "endpoint": ("--endpoint", "str"),
        "region": ("--region", "str"),
        "prefix": ("--prefix", "str"),
        "session_token": ("--session-token", "str"),
    },
    "sftp": {
        "path": ("--path", "str"),
        "host": ("--host", "str"),
        "username": ("--username", "str"),
        "port": ("--port", "str"),
        "keyfile": ("--keyfile", "str"),
        "known_hosts": ("--known-hosts", "str"),
        "sftp_password": ("--sftp-password", "str"),
        "key_data": ("--key-data", "str"),
        "known_hosts_data": ("--known-hosts-data", "str"),
        "embed_credentials": ("--embed-credentials", "bool"),
        "external": ("--external", "bool"),
        "ssh_command": ("--ssh-command", "str"),
        "ssh_args": ("--ssh-args", "list"),
    },
    "webdav": {
        "url": ("--url", "str"),
        "webdav_username": ("--webdav-username", "str"),
        "webdav_password": ("--webdav-password", "str"),
    },
}

# Argument spec entries shared by all kopia modules.
# Include this in each module's argument_spec via dict unpacking.
KOPIA_COMMON_ARGUMENT_SPEC = dict(
    password=dict(type="str", no_log=True),
    config=dict(type="path"),
)


def fmt_backend(value):
    """Format the backend dict into positional + flag arguments for kopia CLI.

    For most providers the output is:
        [provider_name, --flag1=value1, --flag2, ...]

    Param types:
        str  - emits --flag=value; skipped when value is None.
        bool - emits --flag when True; skipped when False or None.
        list - emits --flag item once per item; skipped when empty or None.

    For the "server" provider, returns [] because server connect uses top-level
    flags (--url, --server-cert-fingerprint) passed separately.
    """
    provider = value["provider"]
    if provider == "server":
        return []
    result = [provider]
    for param_name, (flag, kind) in _PROVIDER_BACKEND_MAP[provider].items():
        param_value = value.get(param_name)
        if kind == "str":
            if param_value is not None:
                result.append(f"{flag}={param_value}")
        elif kind == "bool":
            if param_value:
                result.append(flag)
        elif kind == "list":
            if param_value:
                for item in param_value:
                    result.extend([flag, item])
    return result


def _fmt_throttle(value):
    """Format the throttle dict into --flag value arguments for kopia repository throttle set."""
    if not value:
        return []
    flag_map = {
        "download_bytes_per_second": "--download-bytes-per-second",
        "upload_bytes_per_second": "--upload-bytes-per-second",
        "read_requests_per_second": "--read-requests-per-second",
        "write_requests_per_second": "--write-requests-per-second",
        "list_requests_per_second": "--list-requests-per-second",
        "concurrent_reads": "--concurrent-reads",
        "concurrent_writes": "--concurrent-writes",
    }
    result = []
    for param, flag in flag_map.items():
        v = value.get(param)
        if v is not None:
            result.extend([flag, str(v)])
    return result


def kopia_runner(module: AnsibleModule, extra_formats: dict | None = None, **kwargs) -> CmdRunner:
    """Create a CmdRunner for the kopia CLI.

    Provides arg formats for all params shared across kopia modules.
    Pass extra_formats to add module-specific arg formats on top of the shared ones.
    """
    formats = dict(
        cli_action=cmd_runner_fmt.as_list(),
        status=cmd_runner_fmt.as_fixed("repository", "status"),
        get_throttle=cmd_runner_fmt.as_fixed("repository", "throttle", "get"),
        state=cmd_runner_fmt.as_map(STATE_MAP),
        backend=cmd_runner_fmt.as_func(fmt_backend),
        password=cmd_runner_fmt.as_opt_eq_val("--password"),
        fingerprint_tls=cmd_runner_fmt.as_opt_eq_val("--server-cert-fingerprint"),
        url=cmd_runner_fmt.as_opt_eq_val("--url"),
        config=cmd_runner_fmt.as_opt_eq_val("--config-file"),
        throttle_operation=cmd_runner_fmt.as_list(),
        throttle=cmd_runner_fmt.as_func(_fmt_throttle),
    )
    if extra_formats:
        formats.update(extra_formats)
    return CmdRunner(
        module,
        command=["kopia"],
        arg_formats=formats,
        **kwargs,
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_cellular_data_devices_info
short_description: Information module for organizations _devices _cellular _data _devices
description:
  - Get all organizations _devices _cellular _data _devices. - > List devices eligible
    for Cellular Data Management profile assignment in this organization. Returns
    paginated device assignment candidates with current profile, software version,
    modem, and SIM details. Supports filtering by device serials, profile IDs, device
    types, and supported SIM slots.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  includeAssigned:
    description:
      - >
        IncludeAssigned query parameter. Whether to include devices that have already
        been assigned to a Cellular Data Management Profile.
    type: bool
  includedSerials:
    description:
      - >
        IncludedSerials query parameter. List of device serials to force-include in
        the response when the devices would otherwise be filtered out. This override
        is primarily useful for keeping selected devices visible while paging through
        results. Maximum 1000 serials.
    elements: str
    type: list
  excludedSerials:
    description:
      - >
        ExcludedSerials query parameter. List of device serials to force-exclude from
        the response when the devices would otherwise be returned. This override is
        primarily useful for hiding selected devices while paging through results.
        Maximum 1000 serials.
    elements: str
    type: list
  includedProfileIds:
    description:
      - >
        IncludedProfileIds query parameter. List of Cellular Data Management Profile
        IDs to include in the results. Maximum 1000 profile IDs.
    elements: str
    type: list
  excludedProfileIds:
    description:
      - >
        ExcludedProfileIds query parameter. List of Cellular Data Management Profile
        IDs to exclude from the results. Maximum 1000 profile IDs.
    elements: str
    type: list
  deviceTypes:
    description:
      - DeviceTypes query parameter. List of device types to filter by. Maximum 1000
        device types.
    elements: str
    type: list
  slots:
    description:
      - >
        Slots query parameter. List of SIM slot types that devices must support. Accepted
        values are sim1, sim2, and esim. Maximum 3 slots.
    elements: str
    type: list
  name:
    description:
      - Name query parameter. Name of the device to filter by (partial matches allowed).
    type: str
  serials:
    description:
      - Serials query parameter. List of device serials to filter by. Maximum 1000
        serials.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesCellularDataDevices
    description: Complete reference of the getOrganizationDevicesCellularDataDevices
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-cellular-data-devices
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_cellular_data_devices,
  - Paths used are
    get /organizations/{organizationId}/devices/cellular/data/devices,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _cellular _data _devices
  cisco.meraki.organizations_devices_cellular_data_devices_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    includeAssigned: true
    includedSerials: []
    excludedSerials: []
    includedProfileIds: []
    excludedProfileIds: []
    deviceTypes: []
    slots: []
    name: string
    serials: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "name": "string",
          "url": "string",
          "model": "string",
          "software": {
            "currentVersion": {
              "shortName": "string"
            }
          },
          "modems": [
            {
              "index": 0,
              "sims": [
                {
                  "slot": "string",
                  "type": "string",
                  "active": true
                }
              ]
            }
          ],
          "profile": {
            "assigned": true,
            "id": {},
            "name": {}
          },
          "network": {
            "name": "string",
            "id": "string"
          }
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ArgumentSpec,
)
from ansible_collections.community.dns.plugins.module_utils._names import (
    join_labels,
    normalize_label,
    split_into_labels,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    NOT_PROVIDED,
    DNSAPIError,
)

if t.TYPE_CHECKING:  # pragma: no cover
    from .._provider import ProviderInformation
    from .._zone_record_api import NotProvidedType


def create_zone_name_id_argspec(
    provider_information: ProviderInformation,
) -> ArgumentSpec:
    if provider_information.is_zone_id_equal_to_zone_name():
        return ArgumentSpec(
            argument_spec={
                "zone_name": {
                    "type": "str",
                    "aliases": ["zone", "zone_id"],
                    "required": True,
                },
            },
        )
    return ArgumentSpec(
        argument_spec={
            "zone_name": {"type": "str", "aliases": ["zone"]},
            "zone_id": {"type": provider_information.get_zone_id_type()},
        },
        required_one_of=[
            ("zone_name", "zone_id"),
        ],
        mutually_exclusive=[
            ("zone_name", "zone_id"),
        ],
    )


@t.overload
def normalize_dns_name(name: str) -> str: ...  # pragma: no cover


@t.overload
def normalize_dns_name(name: str | None) -> str | None: ...  # pragma: no cover


def normalize_dns_name(name: str | None) -> str | None:
    if name is None:
        return name
    labels, dummy = split_into_labels(name)
    return join_labels([normalize_label(label) for label in labels])


def is_prefix(*, normalized_record: str, normalized_zone: str) -> bool:
    return (
        normalized_record.endswith("." + normalized_zone)
        or normalized_record == normalized_zone
    )


def _extract_prefix(*, normalized_record: str, normalized_zone: str) -> str | None:
    # Assumes that is_prefix() returned True
    if normalized_record == normalized_zone:
        return None
    return normalized_record[: len(normalized_record) - len(normalized_zone) - 1]


def get_prefix(
    normalized_zone: str,
    provider_information: ProviderInformation,
    *,
    normalized_record: str | None = None,
    prefix: str | None = None
) -> tuple[str, str | None]:
    # If normalized_record is not specified, use prefix
    if normalized_record is None:
        if prefix is not None:
            prefix = provider_information.normalize_prefix(normalize_dns_name(prefix))
        return (prefix + "." + normalized_zone) if prefix else normalized_zone, prefix
    # Convert record to prefix
    if not is_prefix(
        normalized_record=normalized_record, normalized_zone=normalized_zone
    ):
        raise DNSAPIError("Record must be in zone")
    return (
        normalized_record,
        _extract_prefix(
            normalized_record=normalized_record, normalized_zone=normalized_zone
        ),
    )


def get_zone_id_or_name(
    module_params: dict[str, t.Any], provider_information: ProviderInformation
) -> tuple[str, None] | tuple[None, t.Any] | tuple[str, t.Any]:
    zone_id_name_equal = provider_information.is_zone_id_equal_to_zone_name()
    zone_name: str | None = module_params["zone_name"]
    zone_id: t.Any | None = None if zone_id_name_equal else module_params["zone_id"]
    if zone_name is not None:
        zone_name = normalize_dns_name(zone_name)
    if zone_id is None:
        assert zone_name is not None
        if zone_id_name_equal:
            return zone_name, zone_name
        return zone_name, None
    return None, zone_id


def get_zone_id_or_name_with_prefix_filter(
    module_params: dict[str, t.Any],
    provider_information: ProviderInformation,
    *,
    use_prefix_or_record: bool = False
) -> tuple[str | None, t.Any | None, str | None | NotProvidedType]:
    zone_name, zone_id = get_zone_id_or_name(module_params, provider_information)
    prefix_filter: str | None | NotProvidedType = NOT_PROVIDED
    if use_prefix_or_record:
        prefix = module_params["prefix"]
        record = module_params["record"]
        if prefix is not None:
            prefix_filter = provider_information.normalize_prefix(
                normalize_dns_name(prefix)
            )
        elif record is not None and zone_name is not None:
            normalized_record = normalize_dns_name(record)
            if is_prefix(
                normalized_record=normalized_record, normalized_zone=zone_name
            ):
                prefix_filter = _extract_prefix(
                    normalized_record=normalized_record, normalized_zone=zone_name
                )
    return zone_name, zone_id, prefix_filter



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The arg spec for the nxos_acls module
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type


class AclsArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_acls module"""

    argument_spec = {
        "config": {
            "elements": "dict",
            "options": {
                "acls": {
                    "elements": "dict",
                    "options": {
                        "aces": {
                            "elements": "dict",
                            "mutually_exclusive": [["grant", "remark"]],
                            "options": {
                                "destination": {
                                    "mutually_exclusive": [
                                        ["address", "any", "host", "prefix"],
                                        [
                                            "wildcard_bits",
                                            "any",
                                            "host",
                                            "prefix",
                                        ],
                                    ],
                                    "options": {
                                        "address": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "port_protocol": {
                                            "mutually_exclusive": [
                                                [
                                                    "eq",
                                                    "lt",
                                                    "neq",
                                                    "gt",
                                                    "range",
                                                ],
                                            ],
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "options": {
                                                        "end": {"type": "str"},
                                                        "start": {"type": "str"},
                                                    },
                                                    "required_together": [["start", "end"]],
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "prefix": {"type": "str"},
                                        "wildcard_bits": {"type": "str"},
                                    },
                                    "required_together": [["address", "wildcard_bits"]],
                                    "type": "dict",
                                },
                                "dscp": {"type": "str"},
                                "fragments": {"type": "bool"},
                                "grant": {
                                    "choices": ["permit", "deny"],
                                    "type": "str",
                                },
                                "log": {"type": "bool"},
                                "precedence": {"type": "str"},
                                "protocol": {"type": "str"},
                                "protocol_options": {
                                    "mutually_exclusive": [["icmp", "igmp", "tcp"]],
                                    "options": {
                                        "icmp": {
                                            "options": {
                                                "administratively_prohibited": {"type": "bool"},
                                                "alternate_address": {"type": "bool"},
                                                "conversion_error": {"type": "bool"},
                                                "dod_host_prohibited": {"type": "bool"},
                                                "dod_net_prohibited": {"type": "bool"},
                                                "echo": {"type": "bool"},
                                                "echo_reply": {"type": "bool"},
                                                "echo_request": {"type": "bool"},
                                                "general_parameter_problem": {"type": "bool"},
                                                "host_isolated": {"type": "bool"},
                                                "host_precedence_unreachable": {"type": "bool"},
                                                "host_redirect": {"type": "bool"},
                                                "host_tos_redirect": {"type": "bool"},
                                                "host_tos_unreachable": {"type": "bool"},
                                                "host_unknown": {"type": "bool"},
                                                "host_unreachable": {"type": "bool"},
                                                "information_reply": {"type": "bool"},
                                                "information_request": {"type": "bool"},
                                                "mask_reply": {"type": "bool"},
                                                "mask_request": {"type": "bool"},
                                                "message_code": {"type": "int"},
                                                "message_type": {"type": "int"},
                                                "mobile_redirect": {"type": "bool"},
                                                "net_redirect": {"type": "bool"},
                                                "net_tos_redirect": {"type": "bool"},
                                                "net_tos_unreachable": {"type": "bool"},
                                                "net_unreachable": {"type": "bool"},
                                                "network_unknown": {"type": "bool"},
                                                "no_room_for_option": {"type": "bool"},
                                                "option_missing": {"type": "bool"},
                                                "packet_too_big": {"type": "bool"},
                                                "parameter_problem": {"type": "bool"},
                                                "port_unreachable": {"type": "bool"},
                                                "precedence_unreachable": {"type": "bool"},
                                                "protocol_unreachable": {"type": "bool"},
                                                "reassembly_timeout": {"type": "bool"},
                                                "redirect": {"type": "bool"},
                                                "router_advertisement": {"type": "bool"},
                                                "router_solicitation": {"type": "bool"},
                                                "source_quench": {"type": "bool"},
                                                "source_route_failed": {"type": "bool"},
                                                "time_exceeded": {"type": "bool"},
                                                "timestamp_reply": {"type": "bool"},
                                                "timestamp_request": {"type": "bool"},
                                                "traceroute": {"type": "bool"},
                                                "ttl_exceeded": {"type": "bool"},
                                                "unreachable": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "icmpv6": {
                                            "type": "dict",
                                            "options": {
                                                "beyond_scope": {"type": "bool"},
                                                "destination_unreachable": {
                                                    "type": "bool",
                                                },
                                                "echo_reply": {"type": "bool"},
                                                "echo_request": {"type": "bool"},
                                                "fragments": {"type": "bool"},
                                                "header": {"type": "bool"},
                                                "hop_limit": {"type": "bool"},
                                                "mld_query": {"type": "bool"},
                                                "mld_reduction": {"type": "bool"},
                                                "mld_report": {"type": "bool"},
                                                "mldv2": {"type": "bool"},
                                                "nd_na": {"type": "bool"},
                                                "nd_ns": {"type": "bool"},
                                                "next_header": {"type": "bool"},
                                                "no_admin": {"type": "bool"},
                                                "no_route": {"type": "bool"},
                                                "packet_too_big": {"type": "bool"},
                                                "parameter_option": {
                                                    "type": "bool",
                                                },
                                                "parameter_problem": {
                                                    "type": "bool",
                                                },
                                                "port_unreachable": {
                                                    "type": "bool",
                                                },
                                                "reassembly_timeout": {
                                                    "type": "bool",
                                                },
                                                "renum_command": {"type": "bool"},
                                                "renum_result": {"type": "bool"},
                                                "renum_seq_number": {
                                                    "type": "bool",
                                                },
                                                "router_advertisement": {
                                                    "type": "bool",
                                                },
                                                "router_renumbering": {
                                                    "type": "bool",
                                                },
                                                "router_solicitation": {
                                                    "type": "bool",
                                                },
                                                "time_exceeded": {"type": "bool"},
                                                "unreachable": {"type": "bool"},
                                                "telemetry_path": {"type": "bool"},
                                                "telemetry_queue": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "igmp": {
                                            "mutually_exclusive": [
                                                [
                                                    "dvmrp",
                                                    "host_query",
                                                    "host_report",
                                                ],
                                            ],
                                            "options": {
                                                "dvmrp": {"type": "bool"},
                                                "host_query": {"type": "bool"},
                                                "host_report": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "tcp": {
                                            "options": {
                                                "ack": {"type": "bool"},
                                                "established": {"type": "bool"},
                                                "fin": {"type": "bool"},
                                                "psh": {"type": "bool"},
                                                "rst": {"type": "bool"},
                                                "syn": {"type": "bool"},
                                                "urg": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "remark": {"type": "str"},
                                "sequence": {"type": "int"},
                                "source": {
                                    "mutually_exclusive": [
                                        ["address", "any", "host", "prefix"],
                                        [
                                            "wildcard_bits",
                                            "host",
                                            "any",
                                            "prefix",
                                        ],
                                    ],
                                    "options": {
                                        "address": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "port_protocol": {
                                            "mutually_exclusive": [
                                                ["eq", "lt", "neq", "range"],
                                                ["eq", "gt", "neq", "range"],
                                            ],
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "options": {
                                                        "end": {"type": "str"},
                                                        "start": {"type": "str"},
                                                    },
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "prefix": {"type": "str"},
                                        "wildcard_bits": {"type": "str"},
                                    },
                                    "required_together": [["address", "wildcard_bits"]],
                                    "type": "dict",
                                },
                            },
                            "type": "list",
                        },
                        "name": {"required": True, "type": "str"},
                    },
                    "type": "list",
                },
                "afi": {
                    "choices": ["ipv4", "ipv6"],
                    "required": True,
                    "type": "str",
                },
            },
            "type": "list",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "deleted",
                "gathered",
                "merged",
                "overridden",
                "rendered",
                "replaced",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
#
# Copyright (c) 2026 Microsoft
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_cognitiveservicesaccount
version_added: "3.21.0"
short_description: Manage Azure AI Cognitive Services accounts
description:
    - Create, update, or delete Azure AI Cognitive Services accounts.
options:
    resource_group:
        description:
            - Name of the resource group.
        required: true
        type: str
        aliases:
            - resource_group_name
    name:
        description:
            - Name of the Cognitive Services account.
            - Must be 2-64 characters, alphanumeric, hyphens, underscores and dots.
            - Must be globally unique.
        required: true
        type: str
    kind:
        description:
            - The API name of cognitive services account.
        type: str
        choices:
            - AIServices
            - CognitiveServices
            - ComputerVision
            - ContentModerator
            - ContentSafety
            - ConversationalLanguageUnderstanding
            - CustomVision.Prediction
            - CustomVision.Training
            - Face
            - FormRecognizer
            - HealthInsights
            - ImmersiveReader
            - Internal.AllInOne
            - LUIS.Authoring
            - LanguageAuthoring
            - MetricsAdvisor
            - OpenAI
            - Personalizer
            - QnAMaker.v2
            - SpeechServices
            - TextAnalytics
            - TextTranslation
    state:
        description:
            - Assert the state of the account.
            - Use C(present) to create or update an account.
            - Use C(absent) to delete an account.
        default: present
        type: str
        choices:
            - present
            - absent
    location:
        description:
            - Valid Azure location.
            - Defaults to location of the resource group.
        type: str
    sku:
        description:
            - The SKU/pricing tier of the Cognitive Services account.
            - C(F0) is the free tier with rate limits.
            - C(S0) is the standard tier.
            - If not specified on Create F0 is the default.
        type: str
        choices:
            - F0
            - S0
    custom_domain_name:
        description:
            - Optional custom subdomain name for token-based authentication.
        type: str
    public_network_access:
        description:
            - Whether public network access is allowed.
            - Defaults to Enabled on Creation.
        type: str
        choices:
            - Enabled
            - Disabled
    disable_local_auth:
        description:
            - Disable key-based authentication, require Microsoft Entra ID.
            - Defaults to False on Creation.
        type: bool
    network_acls:
        description:
            - Network firewall and virtual network rules.
        type: dict
        suboptions:
            default_action:
                description:
                    - Default firewall action.
                type: str
                choices:
                    - Allow
                    - Deny
                default: Allow
            ip_rules:
                description:
                    - List of allowed IP addresses or CIDR ranges.
                type: list
                elements: dict
                suboptions:
                    value:
                        description:
                            - IP address or CIDR range (e.g., '203.0.113.0/24').
                        type: str
                        required: true
            virtual_network_rules:
                description:
                    - List of allowed virtual network subnets.
                type: list
                elements: dict
                suboptions:
                    id:
                        description:
                            - Full resource ID of the subnet.
                        type: str
                        required: true
    purge:
        description:
            - When I(state) is C(absent) setting this to true
              will purge the account as well.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
    - azure.azcollection.azure_identity_multiple_user

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create Cognitive Services account with free tier
  azure.azcollection.azure_rm_cognitiveservicesaccount:
    resource_group: myResourceGroup
    name: mycontentsafety
    kind: ContentSafety
    location: eastus
    sku: F0
    tags:
      purpose: content-moderation

- name: Create Cognitive Services account with network security
  azure.azcollection.azure_rm_cognitiveservicesaccount:
    resource_group: myResourceGroup
    name: mycontentsafety-prod
    kind: ContentSafety
    sku: S0
    public_network_access: Disabled
    network_acls:
      default_action: Deny
      ip_rules:
        - value: "203.0.113.0/24"
    tags:
      environment: production

- name: Create Cognitive Services account with managed identity
  azure.azcollection.azure_rm_cognitiveservicesaccount:
    resource_group: myResourceGroup
    name: mycontentsafety-identity
    kind: ContentSafety
    identity:
      type: SystemAssigned
    disable_local_auth: true

- name: Update tags on existing account
  azure.azcollection.azure_rm_cognitiveservicesaccount:
    resource_group: myResourceGroup
    name: mycontentsafety
    kind: ContentSafety
    tags:
      environment: production

- name: Delete Cognitive Services account
  azure.azcollection.azure_rm_cognitiveservicesaccount:
    resource_group: myResourceGroup
    name: mycontentsafety
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the Cognitive Services account.
    returned: always
    type: dict
    sample: {
      "etag": "\\"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx\\"",
      "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-csa/providers/Microsoft.CognitiveServices/accounts/csexxxxxxxxxxxxxxx",
      "identity": {
          "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "type": "SystemAssigned"
      },
      "kind": "ContentSafety",
      "location": "eastus",
      "name": "csexxxxxxxxxxxxxxx",
      "properties": {
          "allow_project_management": false,
          "call_rate_limit": {
              "rules": [
                  {
                      "count": 10.0,
                      "dynamic_throttling_enabled": true,
                      "key": "ContentSafety.GPTFeature",
                      "match_patterns": [
                          {
                              "method": "*",
                              "path": "contentsafety/text:detectungroundedness*"
                          }
                      ],
                      "renewal_period": 10.0
                  },
                  {
                      "count": 5.0,
                      "key": "ContentSafety.All",
                      "match_patterns": [
                          {
                              "method": "*",
                              "path": "contentsafety/*"
                          }
                      ],
                      "renewal_period": 1.0
                  },
                  {
                      "count": 5.0,
                      "key": "default",
                      "match_patterns": [
                          {
                              "method": "*",
                              "path": "*"
                          }
                      ],
                      "renewal_period": 1.0
                  }
              ]
          },
          "capabilities": [
              {
                  "name": "VirtualNetworks"
              },
              {
                  "name": "EnabledApimVnet",
                  "value": "true"
              },
              {
                  "name": "Container",
                  "value": "ContentSafety.TextAnalyze,ContentSafety.ImageAnalyze,ContentSafety.Jailbreak,ContentSafety.jailbreakanalyze,ContentSafety.jailbreakanalyzenew"
              }
          ],
          "date_created": "2026-06-24T18:41:21.7869275Z",
          "disable_local_auth": true,
          "endpoint": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/",
          "endpoints": {
              "Container": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/",
              "Content Safety": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/"
          },
          "internal_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
          "is_migrated": false,
          "private_endpoint_connections": [],
          "provisioning_state": "Succeeded",
          "public_network_access": "Enabled",
          "quota_limit": {
              "rules": [
                  {
                      "count": 5000.0,
                      "key": "ContentSafety.All",
                      "match_patterns": [
                          {
                              "method": "*",
                              "path": "contentsafety/*"
                          }
                      ],
                      "renewal_period": 2592000.0
                  }
              ]
          }
      },
      "sku": {
          "name": "F0"
      },
      "system_data": {
          "created_at": "2026-06-24T18:41:21.200238Z",
          "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "created_by_type": "Application",
          "last_modified_at": "2026-06-24T18:41:21.200238Z",
          "last_modified_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "last_modified_by_type": "Application"
      },
      "type": "Microsoft.CognitiveServices/accounts"
    }
'''  # NOQA

import re

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.mgmt.cognitiveservices.models import Identity
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMCognitiveServicesAccount(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str',
                                required=True,
                                aliases=['resource_group_name']),
            name=dict(type='str',
                      required=True
                      ),
            kind=dict(type='str',
                      choices=['AIServices',
                               'CognitiveServices',
                               'ComputerVision',
                               'ContentModerator',
                               'ContentSafety',
                               'ConversationalLanguageUnderstanding',
                               'CustomVision.Prediction',
                               'CustomVision.Training',
                               'Face',
                               'FormRecognizer',
                               'HealthInsights',
                               'ImmersiveReader',
                               'Internal.AllInOne',
                               'LUIS.Authoring',
                               'LanguageAuthoring',
                               'MetricsAdvisor',
                               'OpenAI',
                               'Personalizer',
                               'QnAMaker.v2',
                               'SpeechServices',
                               'TextAnalytics',
                               'TextTranslation']
                      ),
            state=dict(type='str',
                       default='present',
                       choices=['present', 'absent']),
            location=dict(type='str'),
            sku=dict(type='str',
                     choices=['F0', 'S0']),
            custom_domain_name=dict(type='str'),
            public_network_access=dict(type='str',
                                       choices=['Enabled', 'Disabled']),
            disable_local_auth=dict(type='bool'),
            purge=dict(type='bool',
                       default=False),
            network_acls=dict(
                type='dict',
                options=dict(
                    default_action=dict(type='str',
                                        default='Allow',
                                        choices=['Allow', 'Deny']),
                    ip_rules=dict(type='list',
                                  elements='dict',
                                  options=dict(value=dict(type='str',
                                                          required=True
                                                          )
                                               )
                                  ),
                    virtual_network_rules=dict(type='list',
                                               elements='dict',
                                               options=dict(id=dict(type='str',
                                                                    required=True
                                                                    )
                                                            )
                                               )
                )
            ),
            identity=dict(
                type='dict',
                options=self.managed_identity_multiple_user_assigned_spec
            ),
            tags=dict(type='dict')
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.sku = None
        self.custom_domain_name = None
        self.public_network_access = None
        self.disable_local_auth = None
        self.network_acls = None
        self.identity = None
        self.tags = None
        self.purge = None
        self._managed_identity = None

        self.results = dict(changed=False,
                            compare=[])

        super(AzureRMCognitiveServicesAccount, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=True
        )

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": Identity,
                                      "user_assigned": dict
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        # Validate account name
        if not re.match(r'^[a-zA-Z0-9][a-zA-Z0-9_.-]*$', self.name):
            self.module.fail_json(msg="Account name must start with alphanumeric and contain only alphanumeric, hyphens, underscores, and dots")
        if len(self.name) < 2 or len(self.name) > 64:
            self.module.fail_json(msg="Account name must be between 2 and 64 characters")

        # Get existing account
        account = self.get_account()

        if self.state == 'present':
            params = self.build_account_parameters(current_account=account)
            if not account:
                # Create new account
                if not self.check_mode:
                    self.results['state'] = self.create_account(params)
                else:
                    self.results['state'] = dict()
                self.results['changed'] = True
            else:
                # Update existing account
                update_needed = self.check_update_needed(account, params)
                if update_needed:
                    if not self.check_mode:
                        self.results['state'] = self.update_account(params)
                    else:
                        self.results['state'] = account
                    self.results['changed'] = True
                else:
                    self.results['state'] = account
        else:  # state == 'absent'
            if account:
                if not self.check_mode:
                    self.delete_account(account['location'])
                self.results['changed'] = True
            self.results['state'] = dict()

        return self.results

    def get_account(self):
        """Get Cognitive Services account if it exists"""
        self.log('Getting Cognitive Services account {0}'.format(self.name))
        try:
            account_obj = self.cognitive_services_management_client.accounts.get(
                self.resource_group,
                self.name
            )
            return account_obj.as_dict()
        except ResourceNotFoundError:
            self.log('Account {0} not found'.format(self.name))
            return None

    def check_update_needed(self, account, params):
        """Check if account needs to be updated"""

        changed = False

        # Check immutable properties
        if account['location'].lower() != params['location'].lower():
            self.module.fail_json(msg="Cannot change location of existing account")

        # Check SKU
        if account['sku']['name'] != params['sku']['name']:
            # Only allow F0 -> S0 upgrade
            if not (account['sku']['name'] == 'F0' and params['sku']['name'] == 'S0'):
                self.module.fail_json(msg="Cannot change SKU from {0} to {1}".format(account['sku']['name'], params['sku']['name']))
            changed = True

        # Check tags
        update_tags, params['tags'] = self.update_tags(account.get('tags'))
        if update_tags:
            changed = True

        if not self.default_compare({},
                                    params,
                                    account,
                                    '',
                                    self.results):
            changed = True

        # Check identity
        if self.identity:
            update_identity, identity = self.update_managed_identity(
                self.identity,
                account['identity'],
                patch_support=True
            )
            params['identity'] = dict(type=identity.type)
            if identity.user_assigned_identities:
                params['identity']['user_assigned_identities'] = identity.user_assigned_identities
            if update_identity:
                changed = True

        return changed

    def build_account_parameters(self, current_account=None):
        """Build account parameters for create/update"""

        # Properties
        properties = {}
        params = {}

        # Required fields

        if isinstance(current_account, dict):
            # Location can't be changed, but it will be checked
            # in check_update method.
            if self.location:
                params['location'] = self.location
            else:
                params['location'] = current_account['location']
            params['kind'] = current_account['kind']
            if self.sku:
                params['sku'] = {'name': self.sku}
            else:
                params['sku'] = current_account['sku']
            if self.public_network_access:
                properties['public_network_access'] = self.public_network_access
            if self.disable_local_auth is not None:
                properties['disable_local_auth'] = self.disable_local_auth
        else:
            # Get resource group location if location not specified
            if not self.location:
                resource_group = self.get_resource_group(self.resource_group)
                self.location = resource_group.location
            params['location'] = self.location

            if not self.kind:
                self.module.fail_json(msg="kind must be specified when creating")
            params['kind'] = self.kind
            update_tags, params['tags'] = self.update_tags(None)
            params['sku'] = {'name': self.sku or 'F0'}
            properties['public_network_access'] = self.public_network_access or 'Enabled'
            properties['disable_local_auth'] = self.disable_local_auth if self.disable_local_auth is not None else False
            if self.identity:
                update_identity, identity = self.update_managed_identity(
                    self.identity,
                )
                params['identity'] = dict(type=identity.type)
                if identity.user_assigned_identities:
                    params['identity']['user_assigned_identities'] = identity.user_assigned_identities

        if self.custom_domain_name:
            properties['custom_sub_domain_name'] = self.custom_domain_name

        # Network ACLs
        if self.network_acls:
            network_rule_set = {}

            if 'default_action' in self.network_acls:
                network_rule_set['default_action'] = self.network_acls['default_action']

            if 'ip_rules' in self.network_acls and \
                    self.network_acls['ip_rules'] is not None:
                network_rule_set['ip_rules'] = [
                    {'value': rule['value']} for rule in self.network_acls['ip_rules']
                ]

            if 'virtual_network_rules' in self.network_acls and \
                    self.network_acls['virtual_network_rules'] is not None:
                network_rule_set['virtual_network_rules'] = [
                    {'id': rule['id']} for rule in self.network_acls['virtual_network_rules']
                ]

            properties['network_acls'] = network_rule_set

        if properties:
            params['properties'] = properties

        return params

    def create_account(self, params):
        """Create a new Cognitive Services account"""
        self.log('Creating Cognitive Services account {0}'.format(self.name))

        try:
            poller = self.cognitive_services_management_client.accounts.begin_create(
                self.resource_group,
                self.name,
                params
            )
            account_obj = self.get_poller_result(poller)
            return account_obj.as_dict()
        except Exception as exc:
            self.module.fail_json(msg="Failed to create Cognitive Services account: {0}".format(str(exc)))

    def update_account(self, params):
        """Update existing Cognitive Services account"""
        self.log('Updating Cognitive Services account {0}'.format(self.name))

        try:
            poller = self.cognitive_services_management_client.accounts.begin_update(
                self.resource_group,
                self.name,
                params
            )
            account_obj = self.get_poller_result(poller)
            return account_obj.as_dict()
        except Exception as exc:
            self.module.fail_json(msg="Failed to update Cognitive Services account: {0}".format(str(exc)))

    def delete_account(self, location):
        """Delete Cognitive Services account"""
        self.log('Deleting Cognitive Services account {0}'.format(self.name))

        try:
            poller = self.cognitive_services_management_client.accounts.begin_delete(
                self.resource_group,
                self.name
            )
            self.get_poller_result(poller)
        except Exception as exc:
            self.module.fail_json(msg="Failed to delete Cognitive Services account: {0}".format(str(exc)))
        if self.purge:
            try:
                poller = self.cognitive_services_management_client.deleted_accounts.begin_purge(
                    location,
                    self.resource_group,
                    self.name
                )
                self.get_poller_result(poller)
            except Exception as exc:
                self.module.fail_json(msg="Failed to purge Cognitive Services account: {0}".format(str(exc)))


def main():
    AzureRMCognitiveServicesAccount()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: directconnect_link_aggregation_group
version_added: 1.0.0
short_description: Manage Direct Connect LAG bundles
description:
  - Create, delete, or modify a Direct Connect link aggregation group.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_direct_connect_link_aggregation_group).
    The usage did not change.
author:
  - "Sloane Hertel (@s-hertel)"
options:
  state:
    description:
      - The state of the Direct Connect link aggregation group.
    choices:
      - present
      - absent
    type: str
    required: true
  name:
    description:
      - The name of the Direct Connect link aggregation group.
    type: str
  link_aggregation_group_id:
    description:
      - The ID of the Direct Connect link aggregation group.
    type: str
  num_connections:
    description:
      - The number of connections with which to initialize the link aggregation group.
    type: int
  min_links:
    description:
      - The minimum number of physical connections that must be operational for the LAG itself to be operational.
    type: int
  location:
    description:
      - The location of the link aggregation group.
    type: str
  bandwidth:
    description:
      - The bandwidth of the link aggregation group.
    type: str
  force_delete:
    description:
      - This allows the minimum number of links to be set to 0, any hosted connections disassociated,
        and any virtual interfaces associated to the LAG deleted.
    type: bool
    default: false
  connection_id:
    description:
      - A connection ID to link with the link aggregation group upon creation.
    type: str
  delete_with_disassociation:
    description:
      - To be used with I(state=absent) to delete connections after disassociating them with the LAG.
    type: bool
    default: false
  wait:
    description:
      - Whether or not to wait for the operation to complete.
      - May be useful when waiting for virtual interfaces to be deleted.
      - The time to wait can be controlled by setting I(wait_timeout).
    type: bool
    default: false
  wait_timeout:
    description:
      - The duration in seconds to wait if I(wait=true).
    default: 120
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

# create a Direct Connect connection
- community.aws.directconnect_link_aggregation_group:
  state: present
  location: EqDC2
  lag_id: dxlag-xxxxxxxx
  bandwidth: 1Gbps
"""

RETURN = r"""
changed:
  type: str
  description: Whether or not the LAG has changed.
  returned: always
aws_device:
  type: str
  description: The AWS Direct Connection endpoint that hosts the LAG.
  sample: "EqSe2-1bwfvazist2k0"
  returned: when I(state=present)
connections:
  type: list
  description: A list of connections bundled by this LAG.
  sample:
    "connections": [
      {
         "aws_device": "EqSe2-1bwfvazist2k0",
         "bandwidth": "1Gbps",
         "connection_id": "dxcon-fgzjah5a",
         "connection_name": "Requested Connection 1 for Lag dxlag-fgtoh97h",
         "connection_state": "down",
         "lag_id": "dxlag-fgnsp4rq",
         "location": "EqSe2",
         "owner_account": "123456789012",
         "region": "us-west-2"
      }
      ]
  returned: when I(state=present)
connections_bandwidth:
  type: str
  description: The individual bandwidth of the physical connections bundled by the LAG.
  sample: "1Gbps"
  returned: when I(state=present)
lag_id:
  type: str
  description: Unique identifier for the link aggregation group.
  sample: "dxlag-fgnsp4rq"
  returned: when I(state=present)
lag_name:
  type: str
  description: User-provided name for the link aggregation group.
  returned: when I(state=present)
lag_state:
  type: str
  description: State of the LAG.
  sample: "pending"
  returned: when I(state=present)
location:
  type: str
  description: Where the connection is located.
  sample: "EqSe2"
  returned: when I(state=present)
minimum_links:
  type: int
  description: The minimum number of physical connections that must be operational for the LAG itself to be operational.
  returned: when I(state=present)
number_of_connections:
  type: int
  description: The number of physical connections bundled by the LAG.
  returned: when I(state=present)
owner_account:
  type: str
  description: Owner account ID of the LAG.
  returned: when I(state=present)
region:
  type: str
  description: The region in which the LAG exists.
  returned: when I(state=present)
"""

import time
import traceback

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.direct_connect import DirectConnectError
from ansible_collections.amazon.aws.plugins.module_utils.direct_connect import delete_connection
from ansible_collections.amazon.aws.plugins.module_utils.direct_connect import delete_virtual_interface
from ansible_collections.amazon.aws.plugins.module_utils.direct_connect import disassociate_connection_and_lag
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def lag_status(client, lag_id):
    return lag_exists(client, lag_id=lag_id, lag_name=None, verify=False)


def lag_exists(client, lag_id=None, lag_name=None, verify=True):
    """If verify=True, returns the LAG ID or None
    If verify=False, returns the LAG's data (or an empty dict)
    """
    try:
        if lag_id:
            response = client.describe_lags(lagId=lag_id)
        else:
            response = client.describe_lags()
    except botocore.exceptions.ClientError as e:
        if lag_id and verify:
            return False
        elif lag_id:
            return {}
        else:
            failed_op = "Failed to describe DirectConnect link aggregation groups."
        raise DirectConnectError(msg=failed_op, last_traceback=traceback.format_exc(), exception=e)

    match = []  # List of LAG IDs that are exact matches
    lag = []  # List of LAG data that are exact matches

    # look for matching connections
    if len(response.get("lags", [])) == 1 and lag_id:
        if response["lags"][0]["lagState"] != "deleted":
            match.append(response["lags"][0]["lagId"])
            lag.append(response["lags"][0])
    else:
        for each in response.get("lags", []):
            if each["lagState"] != "deleted":
                if not lag_id:
                    if lag_name == each["lagName"]:
                        match.append(each["lagId"])
                else:
                    match.append(each["lagId"])

    # verifying if the connections exists; if true, return connection identifier, otherwise return False
    if verify and len(match) == 1:
        return match[0]
    elif verify:
        return False

    # not verifying if the connection exists; just return current connection info
    else:
        if len(lag) == 1:
            return lag[0]
        else:
            return {}


def create_lag(client, num_connections, location, bandwidth, name, connection_id):
    if not name:
        raise DirectConnectError(
            msg="Failed to create a Direct Connect link aggregation group: name required.",
            last_traceback=None,
            exception="",
        )

    parameters = dict(
        numberOfConnections=num_connections, location=location, connectionsBandwidth=bandwidth, lagName=name
    )
    if connection_id:
        parameters.update(connectionId=connection_id)
    try:
        lag = client.create_lag(**parameters)
    except botocore.exceptions.ClientError as e:
        raise DirectConnectError(
            msg=f"Failed to create DirectConnect link aggregation group {name}",
            last_traceback=traceback.format_exc(),
            exception=e,
        )

    return lag["lagId"]


def delete_lag(client, lag_id):
    try:
        client.delete_lag(lagId=lag_id)
    except botocore.exceptions.ClientError as e:
        raise DirectConnectError(
            msg=f"Failed to delete Direct Connect link aggregation group {lag_id}.",
            last_traceback=traceback.format_exc(),
            exception=e,
        )


@AWSRetry.jittered_backoff(retries=5, delay=2, backoff=2.0, catch_extra_error_codes=["DirectConnectClientException"])
def _update_lag(client, lag_id, lag_name, min_links):
    params = {}
    if min_links:
        params.update(minimumLinks=min_links)
    if lag_name:
        params.update(lagName=lag_name)

    client.update_lag(lagId=lag_id, **params)


def update_lag(client, lag_id, lag_name, min_links, num_connections, wait, wait_timeout):
    start = time.time()

    if min_links and min_links > num_connections:
        raise DirectConnectError(
            msg=f"The number of connections {num_connections} must be greater than the minimum number of links {min_links} to update the LAG {lag_id}",
            last_traceback=None,
            exception=None,
        )

    while True:
        try:
            _update_lag(client, lag_id, lag_name, min_links)
        except botocore.exceptions.ClientError as e:
            if wait and time.time() - start <= wait_timeout:
                continue
            msg = f"Failed to update Direct Connect link aggregation group {lag_id}."
            if "MinimumLinks cannot be set higher than the number of connections" in e.response["Error"]["Message"]:
                msg += f"Unable to set the min number of links to {min_links} while the LAG connections are being requested"
            raise DirectConnectError(msg=msg, last_traceback=traceback.format_exc(), exception=e)
        else:
            break


def lag_changed(current_status, name, min_links):
    """Determines if a modifiable link aggregation group attribute has been modified."""
    return (name and name != current_status["lagName"]) or (min_links and min_links != current_status["minimumLinks"])


def ensure_present(
    client, num_connections, lag_id, lag_name, location, bandwidth, connection_id, min_links, wait, wait_timeout
):
    exists = lag_exists(client, lag_id, lag_name)
    if not exists and lag_id:
        raise DirectConnectError(
            msg=f"The Direct Connect link aggregation group {lag_id} does not exist.",
            last_traceback=None,
            exception="",
        )

    # the connection is found; get the latest state and see if it needs to be updated
    if exists:
        lag_id = exists
        latest_state = lag_status(client, lag_id)
        if lag_changed(latest_state, lag_name, min_links):
            update_lag(client, lag_id, lag_name, min_links, num_connections, wait, wait_timeout)
            return True, lag_id
        return False, lag_id

    # no connection found; create a new one
    else:
        lag_id = create_lag(client, num_connections, location, bandwidth, lag_name, connection_id)
        update_lag(client, lag_id, lag_name, min_links, num_connections, wait, wait_timeout)
        return True, lag_id


def describe_virtual_interfaces(client, lag_id):
    try:
        response = client.describe_virtual_interfaces(connectionId=lag_id)
    except botocore.exceptions.ClientError as e:
        raise DirectConnectError(
            msg=f"Failed to describe any virtual interfaces associated with LAG: {lag_id}",
            last_traceback=traceback.format_exc(),
            exception=e,
        )
    return response.get("virtualInterfaces", [])


def get_connections_and_virtual_interfaces(client, lag_id):
    virtual_interfaces = describe_virtual_interfaces(client, lag_id)
    connections = lag_status(client, lag_id=lag_id).get("connections", [])
    return virtual_interfaces, connections


def disassociate_vis(client, lag_id, virtual_interfaces):
    for vi in virtual_interfaces:
        delete_virtual_interface(client, vi["virtualInterfaceId"])
        try:
            client.delete_virtual_interface(virtualInterfaceId=vi["virtualInterfaceId"])
        except botocore.exceptions.ClientError as e:
            raise DirectConnectError(
                msg=f"Could not delete virtual interface {vi} to delete link aggregation group {lag_id}.",
                last_traceback=traceback.format_exc(),
                exception=e,
            )


def ensure_absent(client, lag_id, lag_name, force_delete, delete_with_disassociation, wait, wait_timeout):
    lag_id = lag_exists(client, lag_id, lag_name)
    if not lag_id:
        return False

    latest_status = lag_status(client, lag_id)

    # determine the associated connections and virtual interfaces to disassociate
    virtual_interfaces, connections = get_connections_and_virtual_interfaces(client, lag_id)

    # If min_links is not 0, there are associated connections, or if there are virtual interfaces, ask for force_delete
    if any((latest_status["minimumLinks"], virtual_interfaces, connections)) and not force_delete:
        raise DirectConnectError(
            msg=(
                "There are a minimum number of links, hosted connections, or associated virtual interfaces for LAG"
                f" {lag_id}. To force deletion of the LAG use delete_force: True (if the LAG has virtual interfaces"
                " they will be deleted). Optionally, to ensure hosted connections are deleted after disassociation use"
                " delete_with_disassociation: True and wait: True (as Virtual Interfaces may take a few moments to"
                " delete)"
            ),
            last_traceback=None,
            exception=None,
        )

    # update min_links to be 0 so we can remove the LAG
    update_lag(client, lag_id, None, 0, len(connections), wait, wait_timeout)

    # if virtual_interfaces and not delete_vi_with_disassociation: Raise failure; can't delete while vi attached
    for connection in connections:
        disassociate_connection_and_lag(client, connection["connectionId"], lag_id)
        if delete_with_disassociation:
            delete_connection(client, connection["connectionId"])

    for vi in virtual_interfaces:
        delete_virtual_interface(client, vi["virtualInterfaceId"])

    start_time = time.time()
    while True:
        try:
            delete_lag(client, lag_id)
        except DirectConnectError as e:
            if (
                ("until its Virtual Interfaces are deleted" in e.exception)
                and (time.time() - start_time < wait_timeout)
                and wait
            ):
                continue
        else:
            return True


def main():
    argument_spec = dict(
        state=dict(required=True, choices=["present", "absent"]),
        name=dict(),
        link_aggregation_group_id=dict(),
        num_connections=dict(type="int"),
        min_links=dict(type="int"),
        location=dict(),
        bandwidth=dict(),
        connection_id=dict(),
        delete_with_disassociation=dict(type="bool", default=False),
        force_delete=dict(type="bool", default=False),
        wait=dict(type="bool", default=False),
        wait_timeout=dict(type="int", default=120),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=[("link_aggregation_group_id", "name")],
        required_if=[("state", "present", ("location", "bandwidth"))],
    )

    try:
        connection = module.client("directconnect")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    state = module.params.get("state")
    response = {}
    try:
        if state == "present":
            changed, lag_id = ensure_present(
                connection,
                num_connections=module.params.get("num_connections"),
                lag_id=module.params.get("link_aggregation_group_id"),
                lag_name=module.params.get("name"),
                location=module.params.get("location"),
                bandwidth=module.params.get("bandwidth"),
                connection_id=module.params.get("connection_id"),
                min_links=module.params.get("min_links"),
                wait=module.params.get("wait"),
                wait_timeout=module.params.get("wait_timeout"),
            )
            response = lag_status(connection, lag_id)
        elif state == "absent":
            changed = ensure_absent(
                connection,
                lag_id=module.params.get("link_aggregation_group_id"),
                lag_name=module.params.get("name"),
                force_delete=module.params.get("force_delete"),
                delete_with_disassociation=module.params.get("delete_with_disassociation"),
                wait=module.params.get("wait"),
                wait_timeout=module.params.get("wait_timeout"),
            )
    except DirectConnectError as e:
        if e.last_traceback:
            module.fail_json(msg=e.msg, exception=e.last_traceback, **camel_dict_to_snake_dict(e.exception))
        else:
            module.fail_json(msg=e.msg)

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(response))


if __name__ == "__main__":
    main()
